from typing import List

import datasets

from mteb.abstasks import AbsTaskRetrieval, MultilingualTask, TaskMetadata

_EVAL_LANGS = {
    "ara-ara": ["ara-Arab", "ara-Arab"],
    "eng-ara": ["eng-Latn", "ara-Arab"],
    "ara-eng": ["ara-Arab", "eng-Latn"],
    "deu-deu": ["deu-Latn", "deu-Latn"],
    "eng-deu": ["eng-Latn", "deu-Latn"],
    "deu-eng": ["deu-Latn", "eng-Latn"],
    "spa-spa": ["spa-Latn", "spa-Latn"],
    "eng-spa": ["eng-Latn", "spa-Latn"],
    "spa-eng": ["spa-Latn", "eng-Latn"],
    "fra-fra": ["fra-Latn", "fra-Latn"],
    "eng-fra": ["eng-Latn", "fra-Latn"],
    "fra-eng": ["fra-Latn", "eng-Latn"],
    "hin-hin": ["hin-Deva", "hin-Deva"],
    "eng-hin": ["eng-Latn", "hin-Deva"],
    "hin-eng": ["hin-Deva", "eng-Latn"],
    "ita-ita": ["ita-Latn", "ita-Latn"],
    "eng-ita": ["eng-Latn", "ita-Latn"],
    "ita-eng": ["ita-Latn", "eng-Latn"],
    "jpn-jpn": ["jpn-Hira", "jpn-Hira"],
    "eng-jpn": ["eng-Latn", "jpn-Hira"],
    "jpn-eng": ["jpn-Hira", "eng-Latn"],
    "kor-kor": ["kor-Hang", "kor-Hang"],
    "eng-kor": ["eng-Latn", "kor-Hang"],
    "kor-eng": ["kor-Hang", "eng-Latn"],
    "pol-pol": ["pol-Latn", "pol-Latn"],
    "eng-pol": ["eng-Latn", "pol-Latn"],
    "pol-eng": ["pol-Latn", "eng-Latn"],
    "por-por": ["por-Latn", "por-Latn"],
    "eng-por": ["eng-Latn", "por-Latn"],
    "por-eng": ["por-Latn", "eng-Latn"],
    "tam-tam": ["tam-Taml", "tam-Taml"],
    "eng-tam": ["eng-Latn", "tam-Taml"],
    "tam-eng": ["tam-Taml", "eng-Latn"],
    "cmn-cmn": ["cmn-Hans", "cmn-Hans"],
    "eng-cmn": ["eng-Latn", "cmn-Hans"],
    "cmn-eng": ["cmn-Hans", "eng-Latn"],
}

_LANG_CONVERSION = {
    "ara": "ar",
    "deu": "de",
    "spa": "es",
    "fra": "fr",
    "hin": "hi",
    "ita": "it",
    "jpn": "ja",
    "kor": "ko",
    "pol": "pl",
    "por": "pt",
    "tam": "ta",
    "cmn": "zh",
    "eng": "en",
}


class XPQARetrieval(AbsTaskRetrieval, MultilingualTask):
    metadata = TaskMetadata(
        name="XPQARetrieval",
        description="XPQARetrieval",
        reference="https://arxiv.org/abs/2305.09249",
        dataset={
            "path": "jinaai/xpqa",
            "revision": "c99d599f0a6ab9b85b065da6f9d94f9cf731679f",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2p",
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS,
        main_score="ndcg_at_10",
        date=("2022-01-01", "2023-07-31"),  # best guess
        form=["written"],
        domains=["Reviews"],
        task_subtypes=["Question answering"],
        license="CDLA-Sharing-1.0",
        socioeconomic_status="mixed",
        annotations_creators="human-annotated",
        dialect=[],
        text_creation="found",
        bibtex_citation="""@inproceedings{shen2023xpqa,
        title={xPQA: Cross-Lingual Product Question Answering in 12 Languages},
        author={Shen, Xiaoyu and Asai, Akari and Byrne, Bill and De Gispert, Adria},
        booktitle={Proceedings of the 61st Annual Meeting of the Association for Computational Linguistics (Volume 5: Industry Track)},
        pages={103--115},
        year={2023}
        }""",
        n_samples={"test": 19801},
        avg_character_length={
            "test": {
                "ara-ara": {
                    "average_document_length": 61.88361204013378,
                    "average_query_length": 29.688,
                    "num_documents": 1495,
                    "num_queries": 750,
                    "average_relevant_docs_per_query": 2.004,
                },
                "eng-ara": {
                    "average_document_length": 125.26940639269407,
                    "average_query_length": 29.688,
                    "num_documents": 1533,
                    "num_queries": 750,
                    "average_relevant_docs_per_query": 2.058666666666667,
                },
                "ara-eng": {
                    "average_document_length": 61.88361204013378,
                    "average_query_length": 39.5188679245283,
                    "num_documents": 1495,
                    "num_queries": 742,
                    "average_relevant_docs_per_query": 2.024258760107817,
                },
                "deu-deu": {
                    "average_document_length": 69.54807692307692,
                    "average_query_length": 55.51827676240209,
                    "num_documents": 1248,
                    "num_queries": 766,
                    "average_relevant_docs_per_query": 1.6318537859007833,
                },
                "eng-deu": {
                    "average_document_length": 115.77118078719145,
                    "average_query_length": 55.51827676240209,
                    "num_documents": 1499,
                    "num_queries": 766,
                    "average_relevant_docs_per_query": 1.9634464751958225,
                },
                "deu-eng": {
                    "average_document_length": 69.54807692307692,
                    "average_query_length": 51.88903394255875,
                    "num_documents": 1248,
                    "num_queries": 766,
                    "average_relevant_docs_per_query": 1.6318537859007833,
                },
                "spa-spa": {
                    "average_document_length": 68.27511591962906,
                    "average_query_length": 46.711223203026485,
                    "num_documents": 1941,
                    "num_queries": 793,
                    "average_relevant_docs_per_query": 2.4489281210592684,
                },
                "eng-spa": {
                    "average_document_length": 123.43698347107438,
                    "average_query_length": 46.711223203026485,
                    "num_documents": 1936,
                    "num_queries": 793,
                    "average_relevant_docs_per_query": 2.472887767969735,
                },
                "spa-eng": {
                    "average_document_length": 68.27511591962906,
                    "average_query_length": 47.21059268600252,
                    "num_documents": 1941,
                    "num_queries": 793,
                    "average_relevant_docs_per_query": 2.4489281210592684,
                },
                "fra-fra": {
                    "average_document_length": 76.99354005167959,
                    "average_query_length": 56.0520694259012,
                    "num_documents": 1548,
                    "num_queries": 749,
                    "average_relevant_docs_per_query": 2.069425901201602,
                },
                "eng-fra": {
                    "average_document_length": 137.31242532855435,
                    "average_query_length": 56.0520694259012,
                    "num_documents": 1674,
                    "num_queries": 749,
                    "average_relevant_docs_per_query": 2.248331108144192,
                },
                "fra-eng": {
                    "average_document_length": 76.99354005167959,
                    "average_query_length": 49.58744993324433,
                    "num_documents": 1548,
                    "num_queries": 749,
                    "average_relevant_docs_per_query": 2.069425901201602,
                },
                "hin-hin": {
                    "average_document_length": 47.20783373301359,
                    "average_query_length": 33.47783783783784,
                    "num_documents": 1251,
                    "num_queries": 925,
                    "average_relevant_docs_per_query": 1.3902702702702703,
                },
                "eng-hin": {
                    "average_document_length": 106.67662682602922,
                    "average_query_length": 33.47783783783784,
                    "num_documents": 1506,
                    "num_queries": 925,
                    "average_relevant_docs_per_query": 1.8054054054054054,
                },
                "hin-eng": {
                    "average_document_length": 47.20783373301359,
                    "average_query_length": 34.98574561403509,
                    "num_documents": 1251,
                    "num_queries": 912,
                    "average_relevant_docs_per_query": 1.4100877192982457,
                },
                "ita-ita": {
                    "average_document_length": 59.778301886792455,
                    "average_query_length": 49.14932126696833,
                    "num_documents": 1272,
                    "num_queries": 663,
                    "average_relevant_docs_per_query": 1.9245852187028658,
                },
                "eng-ita": {
                    "average_document_length": 123.07302075326672,
                    "average_query_length": 49.14932126696833,
                    "num_documents": 1301,
                    "num_queries": 663,
                    "average_relevant_docs_per_query": 1.9849170437405732,
                },
                "ita-eng": {
                    "average_document_length": 59.778301886792455,
                    "average_query_length": 49.040723981900456,
                    "num_documents": 1272,
                    "num_queries": 663,
                    "average_relevant_docs_per_query": 1.9245852187028658,
                },
                "jpn-jpn": {
                    "average_document_length": 41.030605871330415,
                    "average_query_length": 23.296969696969697,
                    "num_documents": 1601,
                    "num_queries": 825,
                    "average_relevant_docs_per_query": 1.9406060606060607,
                },
                "eng-jpn": {
                    "average_document_length": 126.2647564469914,
                    "average_query_length": 23.296969696969697,
                    "num_documents": 1745,
                    "num_queries": 825,
                    "average_relevant_docs_per_query": 2.1187878787878787,
                },
                "jpn-eng": {
                    "average_document_length": 41.030605871330415,
                    "average_query_length": 51.416058394160586,
                    "num_documents": 1601,
                    "num_queries": 822,
                    "average_relevant_docs_per_query": 1.9476885644768855,
                },
                "kor-kor": {
                    "average_document_length": 31.22722159730034,
                    "average_query_length": 21.81804281345566,
                    "num_documents": 889,
                    "num_queries": 654,
                    "average_relevant_docs_per_query": 1.5642201834862386,
                },
                "eng-kor": {
                    "average_document_length": 112.41231822070145,
                    "average_query_length": 21.81804281345566,
                    "num_documents": 1169,
                    "num_queries": 654,
                    "average_relevant_docs_per_query": 1.952599388379205,
                },
                "kor-eng": {
                    "average_document_length": 31.22722159730034,
                    "average_query_length": 43.9527687296417,
                    "num_documents": 889,
                    "num_queries": 614,
                    "average_relevant_docs_per_query": 1.6661237785016287,
                },
                "pol-pol": {
                    "average_document_length": 50.66814439518683,
                    "average_query_length": 53.72101910828025,
                    "num_documents": 1579,
                    "num_queries": 785,
                    "average_relevant_docs_per_query": 2.080254777070064,
                },
                "eng-pol": {
                    "average_document_length": 112.96919566457501,
                    "average_query_length": 53.72101910828025,
                    "num_documents": 1753,
                    "num_queries": 785,
                    "average_relevant_docs_per_query": 2.385987261146497,
                },
                "pol-eng": {
                    "average_document_length": 50.66814439518683,
                    "average_query_length": 54.1994851994852,
                    "num_documents": 1579,
                    "num_queries": 777,
                    "average_relevant_docs_per_query": 2.101673101673102,
                },
                "por-por": {
                    "average_document_length": 75.9845869297164,
                    "average_query_length": 42.58875,
                    "num_documents": 1622,
                    "num_queries": 800,
                    "average_relevant_docs_per_query": 2.14,
                },
                "eng-por": {
                    "average_document_length": 111.42525930445393,
                    "average_query_length": 42.58875,
                    "num_documents": 1639,
                    "num_queries": 800,
                    "average_relevant_docs_per_query": 2.21875,
                },
                "por-eng": {
                    "average_document_length": 75.9845869297164,
                    "average_query_length": 46.57967377666248,
                    "num_documents": 1622,
                    "num_queries": 797,
                    "average_relevant_docs_per_query": 2.148055207026349,
                },
                "tam-tam": {
                    "average_document_length": 64.89019607843137,
                    "average_query_length": 33.267263427109974,
                    "num_documents": 1275,
                    "num_queries": 782,
                    "average_relevant_docs_per_query": 1.6994884910485935,
                },
                "eng-tam": {
                    "average_document_length": 96.96361185983828,
                    "average_query_length": 33.267263427109974,
                    "num_documents": 1484,
                    "num_queries": 782,
                    "average_relevant_docs_per_query": 2.0255754475703327,
                },
                "tam-eng": {
                    "average_document_length": 64.89019607843137,
                    "average_query_length": 34.777633289986994,
                    "num_documents": 1275,
                    "num_queries": 769,
                    "average_relevant_docs_per_query": 1.728218465539662,
                },
                "cmn-cmn": {
                    "average_document_length": 20.958944281524925,
                    "average_query_length": 12.21116504854369,
                    "num_documents": 1705,
                    "num_queries": 824,
                    "average_relevant_docs_per_query": 2.0716019417475726,
                },
                "eng-cmn": {
                    "average_document_length": 108.31593874078276,
                    "average_query_length": 12.21116504854369,
                    "num_documents": 1763,
                    "num_queries": 824,
                    "average_relevant_docs_per_query": 2.2633495145631066,
                },
                "cmn-eng": {
                    "average_document_length": 20.958944281524925,
                    "average_query_length": 41.24390243902439,
                    "num_documents": 1705,
                    "num_queries": 820,
                    "average_relevant_docs_per_query": 2.0817073170731706,
                },
            }
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        path = self.metadata_dict["dataset"]["path"]
        revision = self.metadata_dict["dataset"]["revision"]
        eval_splits = self.metadata_dict["eval_splits"]
        dataset = _load_dataset_csv(path, revision, eval_splits)

        self.queries, self.corpus, self.relevant_docs = {}, {}, {}
        for lang_pair, _ in self.metadata.eval_langs.items():
            lang_corpus, lang_question = (
                lang_pair.split("-")[0],
                lang_pair.split("-")[1],
            )
            lang_not_english = lang_corpus if lang_corpus != "eng" else lang_question
            dataset_language = dataset.filter(
                lambda x: x["lang"] == _LANG_CONVERSION.get(lang_not_english)
            )
            question_key = "question_en" if lang_question == "eng" else "question"
            corpus_key = "candidate" if lang_corpus == "eng" else "answer"

            queries_to_ids = {
                eval_split: {
                    q: str(_id)
                    for _id, q in enumerate(
                        set(dataset_language[eval_split][question_key])
                    )
                }
                for eval_split in eval_splits
            }

            self.queries[lang_pair] = {
                eval_split: {v: k for k, v in queries_to_ids[eval_split].items()}
                for eval_split in eval_splits
            }

            corpus_to_ids = {
                eval_split: {
                    document: str(_id)
                    for _id, document in enumerate(
                        set(dataset_language[eval_split][corpus_key])
                    )
                }
                for eval_split in eval_splits
            }

            self.corpus[lang_pair] = {
                eval_split: {
                    v: {"text": k} for k, v in corpus_to_ids[eval_split].items()
                }
                for eval_split in eval_splits
            }

            self.relevant_docs[lang_pair] = {}
            for eval_split in eval_splits:
                self.relevant_docs[lang_pair][eval_split] = {}
                for example in dataset_language[eval_split]:
                    query_id = queries_to_ids[eval_split].get(example[question_key])
                    document_id = corpus_to_ids[eval_split].get(example[corpus_key])
                    if query_id in self.relevant_docs[lang_pair][eval_split]:
                        self.relevant_docs[lang_pair][eval_split][query_id][
                            document_id
                        ] = 1
                    else:
                        self.relevant_docs[lang_pair][eval_split][query_id] = {
                            document_id: 1
                        }

        self.data_loaded = True


def _load_dataset_csv(path: str, revision: str, eval_splits: List[str]):
    data_files = {
        eval_split: f"https://huggingface.co/datasets/{path}/resolve/{revision}/{eval_split}.csv"
        for eval_split in eval_splits
    }
    dataset = datasets.load_dataset("csv", data_files=data_files)
    dataset = dataset.filter(lambda x: x["answer"] is not None)

    return dataset



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class TempReasonL2Fact(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="TempReasonL2Fact",
        description="Measuring the ability to retrieve the groundtruth answers to reasoning task queries on TempReason l2-fact.",
        reference="https://github.com/DAMO-NLP-SG/TempReason",
        dataset={
            "path": "RAR-b/TempReason-l2-fact",
            "revision": "13758bcf978613b249d0de4d0840f57815122bdf",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2s",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2023-01-01", "2023-12-31"),
        form=["written"],
        domains=["Encyclopaedic"],
        task_subtypes=["Reasoning as Retrieval"],
        license="CC BY-SA 3.0",
        socioeconomic_status="medium",
        annotations_creators="derived",
        dialect=[],
        text_creation="found",
        bibtex_citation="""@article{xiao2024rar,
  title={RAR-b: Reasoning as Retrieval Benchmark},
  author={Xiao, Chenghao and Hudson, G Thomas and Moubayed, Noura Al},
  journal={arXiv preprint arXiv:2404.06347},
  year={2024}
}
@article{tan2023towards,
  title={Towards benchmarking and improving the temporal reasoning capability of large language models},
  author={Tan, Qingyu and Ng, Hwee Tou and Bing, Lidong},
  journal={arXiv preprint arXiv:2306.08952},
  year={2023}
}
""",
        n_samples={"test": 5397},
        avg_character_length={
            "test": {
                "average_document_length": 19.823525685690758,
                "average_query_length": 830.7268853066519,
                "num_documents": 15787,
                "num_queries": 5397,
                "average_relevant_docs_per_query": 1.0,
            }
        },
    )



from typing import Dict, List

import datasets

from mteb.abstasks import AbsTaskRetrieval, MultilingualTask, TaskMetadata

_LANGUAGES = {
    "mlqa.ar.ar": ["ara-Arab", "ara-Arab"],
    "mlqa.ar.de": ["ara-Arab", "deu-Latn"],
    "mlqa.ar.en": ["ara-Arab", "eng-Latn"],
    "mlqa.ar.es": ["ara-Arab", "spa-Latn"],
    "mlqa.ar.hi": ["ara-Arab", "hin-Deva"],
    "mlqa.ar.vi": ["ara-Arab", "vie-Latn"],
    "mlqa.ar.zh": ["ara-Arab", "zho-Hans"],
    "mlqa.de.ar": ["deu-Latn", "ara-Arab"],
    "mlqa.de.de": ["deu-Latn", "deu-Latn"],
    "mlqa.de.en": ["deu-Latn", "eng-Latn"],
    "mlqa.de.es": ["deu-Latn", "spa-Latn"],
    "mlqa.de.hi": ["deu-Latn", "hin-Deva"],
    "mlqa.de.vi": ["deu-Latn", "vie-Latn"],
    "mlqa.de.zh": ["deu-Latn", "zho-Hans"],
    "mlqa.en.ar": ["eng-Latn", "ara-Arab"],
    "mlqa.en.de": ["eng-Latn", "deu-Latn"],
    "mlqa.en.en": ["eng-Latn", "eng-Latn"],
    "mlqa.en.es": ["eng-Latn", "spa-Latn"],
    "mlqa.en.hi": ["eng-Latn", "hin-Deva"],
    "mlqa.en.vi": ["eng-Latn", "vie-Latn"],
    "mlqa.en.zh": ["eng-Latn", "zho-Hans"],
    "mlqa.es.ar": ["spa-Latn", "ara-Arab"],
    "mlqa.es.de": ["spa-Latn", "deu-Latn"],
    "mlqa.es.en": ["spa-Latn", "eng-Latn"],
    "mlqa.es.es": ["spa-Latn", "spa-Latn"],
    "mlqa.es.hi": ["spa-Latn", "hin-Deva"],
    "mlqa.es.vi": ["spa-Latn", "vie-Latn"],
    "mlqa.es.zh": ["spa-Latn", "zho-Hans"],
    "mlqa.hi.ar": ["hin-Deva", "ara-Arab"],
    "mlqa.hi.de": ["hin-Deva", "deu-Latn"],
    "mlqa.hi.en": ["hin-Deva", "eng-Latn"],
    "mlqa.hi.es": ["hin-Deva", "spa-Latn"],
    "mlqa.hi.hi": ["hin-Deva", "hin-Deva"],
    "mlqa.hi.vi": ["hin-Deva", "vie-Latn"],
    "mlqa.hi.zh": ["hin-Deva", "zho-Hans"],
    "mlqa.vi.ar": ["vie-Latn", "ara-Arab"],
    "mlqa.vi.de": ["vie-Latn", "deu-Latn"],
    "mlqa.vi.en": ["vie-Latn", "eng-Latn"],
    "mlqa.vi.es": ["vie-Latn", "spa-Latn"],
    "mlqa.vi.hi": ["vie-Latn", "hin-Deva"],
    "mlqa.vi.vi": ["vie-Latn", "vie-Latn"],
    "mlqa.vi.zh": ["vie-Latn", "zho-Hans"],
    "mlqa.zh.ar": ["zho-Hans", "ara-Arab"],
    "mlqa.zh.de": ["zho-Hans", "deu-Latn"],
    "mlqa.zh.en": ["zho-Hans", "eng-Latn"],
    "mlqa.zh.es": ["zho-Hans", "spa-Latn"],
    "mlqa.zh.hi": ["zho-Hans", "hin-Deva"],
    "mlqa.zh.vi": ["zho-Hans", "vie-Latn"],
    "mlqa.zh.zh": ["zho-Hans", "zho-Hans"],
}


def _build_lang_pair(langs: List[str]) -> str:
    """Builds a language pair separated by a dash.
    e.g., ['eng-Latn', 'deu-Latn'] -> 'eng-deu'.
    """
    return langs[0].split("-")[0] + "-" + langs[1].split("-")[0]


def extend_lang_pairs() -> Dict[str, List[str]]:
    eval_langs = {}
    for langs in _LANGUAGES.values():
        lang_pair = _build_lang_pair(langs)
        eval_langs[lang_pair] = langs
    return eval_langs


_EVAL_LANGS = extend_lang_pairs()


class MLQARetrieval(AbsTaskRetrieval, MultilingualTask):
    metadata = TaskMetadata(
        name="MLQARetrieval",
        description="""MLQA (MultiLingual Question Answering) is a benchmark dataset for evaluating cross-lingual question answering performance.
        MLQA consists of over 5K extractive QA instances (12K in English) in SQuAD format in seven languages - English, Arabic,
        German, Spanish, Hindi, Vietnamese and Simplified Chinese. MLQA is highly parallel, with QA instances parallel between
        4 different languages on average.""",
        reference="https://huggingface.co/datasets/mlqa",
        dataset={
            "path": "facebook/mlqa",
            "revision": "397ed406c1a7902140303e7faf60fff35b58d285",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2p",
        eval_splits=["validation", "test"],
        eval_langs=_EVAL_LANGS,
        main_score="ndcg_at_10",
        date=("2019-01-01", "2020-12-31"),
        form=["written"],
        domains=["Encyclopaedic"],
        task_subtypes=["Question answering"],
        license="cc-by-sa-3.0",
        socioeconomic_status="mixed",
        annotations_creators="human-annotated",
        dialect=[],
        text_creation="found",
        bibtex_citation="""@article{lewis2019mlqa,
        title = {MLQA: Evaluating Cross-lingual Extractive Question Answering},
        author = {Lewis, Patrick and Oguz, Barlas and Rinott, Ruty and Riedel, Sebastian and Schwenk, Holger},
        journal = {arXiv preprint arXiv:1910.07475},
        year = 2019,
        eid = {arXiv: 1910.07475}
        }""",
        n_samples={"test": 158083, "validation": 15747},
        avg_character_length={
            "validation": {
                "ara-ara": {
                    "average_document_length": 693.8883826879271,
                    "average_query_length": 42.321083172147,
                    "num_documents": 439,
                    "num_queries": 517,
                    "average_relevant_docs_per_query": 1.0,
                },
                "ara-deu": {
                    "average_document_length": 759.3882352941176,
                    "average_query_length": 55.14492753623188,
                    "num_documents": 170,
                    "num_queries": 207,
                    "average_relevant_docs_per_query": 1.0,
                },
                "ara-eng": {
                    "average_document_length": 693.8883826879271,
                    "average_query_length": 50.029013539651835,
                    "num_documents": 439,
                    "num_queries": 517,
                    "average_relevant_docs_per_query": 1.0,
                },
                "ara-spa": {
                    "average_document_length": 654.3071428571428,
                    "average_query_length": 53.68944099378882,
                    "num_documents": 140,
                    "num_queries": 161,
                    "average_relevant_docs_per_query": 1.0,
                },
                "ara-hin": {
                    "average_document_length": 626.5935483870968,
                    "average_query_length": 51.956989247311824,
                    "num_documents": 155,
                    "num_queries": 186,
                    "average_relevant_docs_per_query": 1.0,
                },
                "ara-vie": {
                    "average_document_length": 804.6216216216217,
                    "average_query_length": 49.57055214723926,
                    "num_documents": 148,
                    "num_queries": 163,
                    "average_relevant_docs_per_query": 1.0,
                },
                "ara-zho": {
                    "average_document_length": 787.3161290322581,
                    "average_query_length": 15.617021276595745,
                    "num_documents": 155,
                    "num_queries": 188,
                    "average_relevant_docs_per_query": 1.0,
                },
                "deu-ara": {
                    "average_document_length": 702.1675977653631,
                    "average_query_length": 43.06280193236715,
                    "num_documents": 179,
                    "num_queries": 207,
                    "average_relevant_docs_per_query": 1.0,
                },
                "deu-deu": {
                    "average_document_length": 721.405701754386,
                    "average_query_length": 52.572265625,
                    "num_documents": 456,
                    "num_queries": 512,
                    "average_relevant_docs_per_query": 1.0,
                },
                "deu-eng": {
                    "average_document_length": 721.405701754386,
                    "average_query_length": 48.33984375,
                    "num_documents": 456,
                    "num_queries": 512,
                    "average_relevant_docs_per_query": 1.0,
                },
                "deu-spa": {
                    "average_document_length": 677.2762430939226,
                    "average_query_length": 50.60204081632653,
                    "num_documents": 181,
                    "num_queries": 196,
                    "average_relevant_docs_per_query": 1.0,
                },
                "deu-hin": {
                    "average_document_length": 685.917808219178,
                    "average_query_length": 47.01840490797546,
                    "num_documents": 146,
                    "num_queries": 163,
                    "average_relevant_docs_per_query": 1.0,
                },
                "deu-vie": {
                    "average_document_length": 921.6196319018405,
                    "average_query_length": 46.81868131868132,
                    "num_documents": 163,
                    "num_queries": 182,
                    "average_relevant_docs_per_query": 1.0,
                },
                "deu-zho": {
                    "average_document_length": 736.6347305389221,
                    "average_query_length": 14.936842105263159,
                    "num_documents": 167,
                    "num_queries": 190,
                    "average_relevant_docs_per_query": 1.0,
                },
                "eng-ara": {
                    "average_document_length": 979.3447488584475,
                    "average_query_length": 42.321083172147,
                    "num_documents": 438,
                    "num_queries": 517,
                    "average_relevant_docs_per_query": 1.0,
                },
                "eng-deu": {
                    "average_document_length": 947.3109619686801,
                    "average_query_length": 52.572265625,
                    "num_documents": 447,
                    "num_queries": 512,
                    "average_relevant_docs_per_query": 1.0,
                },
                "eng-eng": {
                    "average_document_length": 940.2842535787321,
                    "average_query_length": 49.01480836236934,
                    "num_documents": 978,
                    "num_queries": 1148,
                    "average_relevant_docs_per_query": 1.0,
                },
                "eng-spa": {
                    "average_document_length": 904.3166287015945,
                    "average_query_length": 52.146,
                    "num_documents": 439,
                    "num_queries": 500,
                    "average_relevant_docs_per_query": 1.0,
                },
                "eng-hin": {
                    "average_document_length": 926.9621749408983,
                    "average_query_length": 49.3905325443787,
                    "num_documents": 423,
                    "num_queries": 507,
                    "average_relevant_docs_per_query": 1.0,
                },
                "eng-vie": {
                    "average_document_length": 1011.8296460176991,
                    "average_query_length": 48.082191780821915,
                    "num_documents": 452,
                    "num_queries": 511,
                    "average_relevant_docs_per_query": 1.0,
                },
                "eng-zho": {
                    "average_document_length": 1001.5046511627907,
                    "average_query_length": 15.39484126984127,
                    "num_documents": 430,
                    "num_queries": 504,
                    "average_relevant_docs_per_query": 1.0,
                },
                "spa-ara": {
                    "average_document_length": 674.3586206896551,
                    "average_query_length": 41.36024844720497,
                    "num_documents": 145,
                    "num_queries": 161,
                    "average_relevant_docs_per_query": 1.0,
                },
                "spa-deu": {
                    "average_document_length": 544.0489130434783,
                    "average_query_length": 51.86734693877551,
                    "num_documents": 184,
                    "num_queries": 196,
                    "average_relevant_docs_per_query": 1.0,
                },
                "spa-eng": {
                    "average_document_length": 641.8215859030837,
                    "average_query_length": 49.156,
                    "num_documents": 454,
                    "num_queries": 500,
                    "average_relevant_docs_per_query": 1.0,
                },
                "spa-spa": {
                    "average_document_length": 641.8215859030837,
                    "average_query_length": 52.146,
                    "num_documents": 454,
                    "num_queries": 500,
                    "average_relevant_docs_per_query": 1.0,
                },
                "spa-hin": {
                    "average_document_length": 703.3212121212122,
                    "average_query_length": 48.080213903743314,
                    "num_documents": 165,
                    "num_queries": 187,
                    "average_relevant_docs_per_query": 1.0,
                },
                "spa-vie": {
                    "average_document_length": 737.8579545454545,
                    "average_query_length": 48.82539682539682,
                    "num_documents": 176,
                    "num_queries": 189,
                    "average_relevant_docs_per_query": 1.0,
                },
                "spa-zho": {
                    "average_document_length": 605.52,
                    "average_query_length": 15.590062111801242,
                    "num_documents": 150,
                    "num_queries": 161,
                    "average_relevant_docs_per_query": 1.0,
                },
                "hin-ara": {
                    "average_document_length": 670.0394736842105,
                    "average_query_length": 43.623655913978496,
                    "num_documents": 152,
                    "num_queries": 186,
                    "average_relevant_docs_per_query": 1.0,
                },
                "hin-deu": {
                    "average_document_length": 596.9718309859155,
                    "average_query_length": 51.41717791411043,
                    "num_documents": 142,
                    "num_queries": 163,
                    "average_relevant_docs_per_query": 1.0,
                },
                "hin-eng": {
                    "average_document_length": 691.5482352941176,
                    "average_query_length": 49.75936883629191,
                    "num_documents": 425,
                    "num_queries": 507,
                    "average_relevant_docs_per_query": 1.0,
                },
                "hin-spa": {
                    "average_document_length": 718.4904458598726,
                    "average_query_length": 52.75935828877005,
                    "num_documents": 157,
                    "num_queries": 187,
                    "average_relevant_docs_per_query": 1.0,
                },
                "hin-hin": {
                    "average_document_length": 691.5482352941176,
                    "average_query_length": 49.3905325443787,
                    "num_documents": 425,
                    "num_queries": 507,
                    "average_relevant_docs_per_query": 1.0,
                },
                "hin-vie": {
                    "average_document_length": 778.484076433121,
                    "average_query_length": 48.35028248587571,
                    "num_documents": 157,
                    "num_queries": 177,
                    "average_relevant_docs_per_query": 1.0,
                },
                "hin-zho": {
                    "average_document_length": 685.0679012345679,
                    "average_query_length": 15.97883597883598,
                    "num_documents": 162,
                    "num_queries": 189,
                    "average_relevant_docs_per_query": 1.0,
                },
                "vie-ara": {
                    "average_document_length": 886.6052631578947,
                    "average_query_length": 41.214723926380366,
                    "num_documents": 152,
                    "num_queries": 163,
                    "average_relevant_docs_per_query": 1.0,
                },
                "vie-deu": {
                    "average_document_length": 981.4534161490683,
                    "average_query_length": 51.27472527472528,
                    "num_documents": 161,
                    "num_queries": 182,
                    "average_relevant_docs_per_query": 1.0,
                },
                "vie-eng": {
                    "average_document_length": 892.7250554323725,
                    "average_query_length": 48.09001956947162,
                    "num_documents": 451,
                    "num_queries": 511,
                    "average_relevant_docs_per_query": 1.0,
                },
                "vie-spa": {
                    "average_document_length": 936.6746987951807,
                    "average_query_length": 51.851851851851855,
                    "num_documents": 166,
                    "num_queries": 189,
                    "average_relevant_docs_per_query": 1.0,
                },
                "vie-hin": {
                    "average_document_length": 869.0509554140127,
                    "average_query_length": 46.44632768361582,
                    "num_documents": 157,
                    "num_queries": 177,
                    "average_relevant_docs_per_query": 1.0,
                },
                "vie-vie": {
                    "average_document_length": 892.7250554323725,
                    "average_query_length": 48.082191780821915,
                    "num_documents": 451,
                    "num_queries": 511,
                    "average_relevant_docs_per_query": 1.0,
                },
                "vie-zho": {
                    "average_document_length": 960.7349397590361,
                    "average_query_length": 15.048913043478262,
                    "num_documents": 166,
                    "num_queries": 184,
                    "average_relevant_docs_per_query": 1.0,
                },
                "zho-ara": {
                    "average_document_length": 238.75155279503105,
                    "average_query_length": 44.34574468085106,
                    "num_documents": 161,
                    "num_queries": 188,
                    "average_relevant_docs_per_query": 1.0,
                },
                "zho-deu": {
                    "average_document_length": 257.109756097561,
                    "average_query_length": 53.84736842105263,
                    "num_documents": 164,
                    "num_queries": 190,
                    "average_relevant_docs_per_query": 1.0,
                },
                "zho-eng": {
                    "average_document_length": 246.65237020316027,
                    "average_query_length": 50.15079365079365,
                    "num_documents": 443,
                    "num_queries": 504,
                    "average_relevant_docs_per_query": 1.0,
                },
                "zho-spa": {
                    "average_document_length": 249.6081081081081,
                    "average_query_length": 52.857142857142854,
                    "num_documents": 148,
                    "num_queries": 161,
                    "average_relevant_docs_per_query": 1.0,
                },
                "zho-hin": {
                    "average_document_length": 238.5521472392638,
                    "average_query_length": 52.05291005291005,
                    "num_documents": 163,
                    "num_queries": 189,
                    "average_relevant_docs_per_query": 1.0,
                },
                "zho-vie": {
                    "average_document_length": 268.32142857142856,
                    "average_query_length": 49.33695652173913,
                    "num_documents": 168,
                    "num_queries": 184,
                    "average_relevant_docs_per_query": 1.0,
                },
                "zho-zho": {
                    "average_document_length": 246.65237020316027,
                    "average_query_length": 15.39484126984127,
                    "num_documents": 443,
                    "num_queries": 504,
                    "average_relevant_docs_per_query": 1.0,
                },
            },
            "test": {
                "ara-ara": {
                    "average_document_length": 698.5714593198451,
                    "average_query_length": 41.26176636039752,
                    "num_documents": 4646,
                    "num_queries": 5333,
                    "average_relevant_docs_per_query": 1.000375023438965,
                },
                "ara-deu": {
                    "average_document_length": 592.5728542914171,
                    "average_query_length": 51.27730582524272,
                    "num_documents": 1503,
                    "num_queries": 1648,
                    "average_relevant_docs_per_query": 1.0006067961165048,
                },
                "ara-eng": {
                    "average_document_length": 698.5714593198451,
                    "average_query_length": 48.556451612903224,
                    "num_documents": 4646,
                    "num_queries": 5332,
                    "average_relevant_docs_per_query": 1.000562640660165,
                },
                "ara-spa": {
                    "average_document_length": 713.4833239118146,
                    "average_query_length": 51.406471183013146,
                    "num_documents": 1769,
                    "num_queries": 1978,
                    "average_relevant_docs_per_query": 1.0,
                },
                "ara-hin": {
                    "average_document_length": 702.1388888888889,
                    "average_query_length": 48.71818678317859,
                    "num_documents": 1512,
                    "num_queries": 1831,
                    "average_relevant_docs_per_query": 1.0,
                },
                "ara-vie": {
                    "average_document_length": 745.4528096017458,
                    "average_query_length": 48.815828041035665,
                    "num_documents": 1833,
                    "num_queries": 2047,
                    "average_relevant_docs_per_query": 1.0,
                },
                "ara-zho": {
                    "average_document_length": 774.4593639575971,
                    "average_query_length": 14.985355648535565,
                    "num_documents": 1698,
                    "num_queries": 1912,
                    "average_relevant_docs_per_query": 1.0,
                },
                "deu-ara": {
                    "average_document_length": 719.6800267201069,
                    "average_query_length": 39.54578532443905,
                    "num_documents": 1497,
                    "num_queries": 1649,
                    "average_relevant_docs_per_query": 1.0,
                },
                "deu-deu": {
                    "average_document_length": 725.5304712558599,
                    "average_query_length": 51.610680257035234,
                    "num_documents": 4053,
                    "num_queries": 4513,
                    "average_relevant_docs_per_query": 1.0008863283846665,
                },
                "deu-eng": {
                    "average_document_length": 725.5304712558599,
                    "average_query_length": 47.07777531575449,
                    "num_documents": 4053,
                    "num_queries": 4513,
                    "average_relevant_docs_per_query": 1.0008863283846665,
                },
                "deu-spa": {
                    "average_document_length": 740.5414052697616,
                    "average_query_length": 50.098591549295776,
                    "num_documents": 1594,
                    "num_queries": 1775,
                    "average_relevant_docs_per_query": 1.0005633802816902,
                },
                "deu-hin": {
                    "average_document_length": 674.3714063714064,
                    "average_query_length": 45.146153846153844,
                    "num_documents": 1287,
                    "num_queries": 1430,
                    "average_relevant_docs_per_query": 1.0,
                },
                "deu-vie": {
                    "average_document_length": 760.1198945981555,
                    "average_query_length": 46.64358208955224,
                    "num_documents": 1518,
                    "num_queries": 1675,
                    "average_relevant_docs_per_query": 1.0,
                },
                "deu-zho": {
                    "average_document_length": 771.3367697594501,
                    "average_query_length": 14.942592592592593,
                    "num_documents": 1455,
                    "num_queries": 1620,
                    "average_relevant_docs_per_query": 1.0006172839506173,
                },
                "eng-ara": {
                    "average_document_length": 1008.3584455058619,
                    "average_query_length": 41.26176636039752,
                    "num_documents": 4606,
                    "num_queries": 5333,
                    "average_relevant_docs_per_query": 1.000375023438965,
                },
                "eng-deu": {
                    "average_document_length": 910.3226686507936,
                    "average_query_length": 51.610680257035234,
                    "num_documents": 4032,
                    "num_queries": 4513,
                    "average_relevant_docs_per_query": 1.0008863283846665,
                },
                "eng-eng": {
                    "average_document_length": 983.0993344090359,
                    "average_query_length": 47.960714902434816,
                    "num_documents": 9916,
                    "num_queries": 11582,
                    "average_relevant_docs_per_query": 1.000690726990157,
                },
                "eng-spa": {
                    "average_document_length": 967.4622376109068,
                    "average_query_length": 50.923252713768804,
                    "num_documents": 4621,
                    "num_queries": 5251,
                    "average_relevant_docs_per_query": 1.000380879832413,
                },
                "eng-hin": {
                    "average_document_length": 986.0465631929046,
                    "average_query_length": 47.328315703824245,
                    "num_documents": 4059,
                    "num_queries": 4916,
                    "average_relevant_docs_per_query": 1.000406834825061,
                },
                "eng-vie": {
                    "average_document_length": 1048.6062197940744,
                    "average_query_length": 48.094085532302095,
                    "num_documents": 4759,
                    "num_queries": 5495,
                    "average_relevant_docs_per_query": 1.0,
                },
                "eng-zho": {
                    "average_document_length": 1063.8536257833482,
                    "average_query_length": 15.019080996884735,
                    "num_documents": 4468,
                    "num_queries": 5136,
                    "average_relevant_docs_per_query": 1.0001947040498442,
                },
                "spa-ara": {
                    "average_document_length": 645.5182320441988,
                    "average_query_length": 40.78412537917088,
                    "num_documents": 1810,
                    "num_queries": 1978,
                    "average_relevant_docs_per_query": 1.0,
                },
                "spa-deu": {
                    "average_document_length": 586.6057810578105,
                    "average_query_length": 51.870913190529876,
                    "num_documents": 1626,
                    "num_queries": 1774,
                    "average_relevant_docs_per_query": 1.0011273957158964,
                },
                "spa-eng": {
                    "average_document_length": 630.6735979836169,
                    "average_query_length": 47.827907862173994,
                    "num_documents": 4761,
                    "num_queries": 5253,
                    "average_relevant_docs_per_query": 1.0,
                },
                "spa-spa": {
                    "average_document_length": 630.6735979836169,
                    "average_query_length": 50.923252713768804,
                    "num_documents": 4761,
                    "num_queries": 5251,
                    "average_relevant_docs_per_query": 1.000380879832413,
                },
                "spa-hin": {
                    "average_document_length": 613.3478260869565,
                    "average_query_length": 46.36680208937899,
                    "num_documents": 1518,
                    "num_queries": 1723,
                    "average_relevant_docs_per_query": 1.0,
                },
                "spa-vie": {
                    "average_document_length": 659.6179295624333,
                    "average_query_length": 48.1595639246779,
                    "num_documents": 1874,
                    "num_queries": 2018,
                    "average_relevant_docs_per_query": 1.0,
                },
                "spa-zho": {
                    "average_document_length": 668.6646171045277,
                    "average_query_length": 15.115562403697997,
                    "num_documents": 1789,
                    "num_queries": 1947,
                    "average_relevant_docs_per_query": 1.0,
                },
                "hin-ara": {
                    "average_document_length": 765.0352862849534,
                    "average_query_length": 42.04642271982523,
                    "num_documents": 1502,
                    "num_queries": 1831,
                    "average_relevant_docs_per_query": 1.0,
                },
                "hin-deu": {
                    "average_document_length": 719.676862745098,
                    "average_query_length": 51.002799160251925,
                    "num_documents": 1275,
                    "num_queries": 1429,
                    "average_relevant_docs_per_query": 1.000699790062981,
                },
                "hin-eng": {
                    "average_document_length": 760.9956086850451,
                    "average_query_length": 47.91232709519935,
                    "num_documents": 4099,
                    "num_queries": 4916,
                    "average_relevant_docs_per_query": 1.000406834825061,
                },
                "hin-spa": {
                    "average_document_length": 753.5010281014394,
                    "average_query_length": 50.46689895470383,
                    "num_documents": 1459,
                    "num_queries": 1722,
                    "average_relevant_docs_per_query": 1.0005807200929153,
                },
                "hin-hin": {
                    "average_document_length": 760.9956086850451,
                    "average_query_length": 47.328315703824245,
                    "num_documents": 4099,
                    "num_queries": 4916,
                    "average_relevant_docs_per_query": 1.000406834825061,
                },
                "hin-vie": {
                    "average_document_length": 789.9253822629969,
                    "average_query_length": 48.21160760143811,
                    "num_documents": 1635,
                    "num_queries": 1947,
                    "average_relevant_docs_per_query": 1.0,
                },
                "hin-zho": {
                    "average_document_length": 834.2057448229793,
                    "average_query_length": 15.101301641199774,
                    "num_documents": 1497,
                    "num_queries": 1767,
                    "average_relevant_docs_per_query": 1.0,
                },
                "vie-ara": {
                    "average_document_length": 992.2129527991218,
                    "average_query_length": 41.82462139716659,
                    "num_documents": 1822,
                    "num_queries": 2047,
                    "average_relevant_docs_per_query": 1.0,
                },
                "vie-deu": {
                    "average_document_length": 861.0610079575597,
                    "average_query_length": 51.58721624850657,
                    "num_documents": 1508,
                    "num_queries": 1674,
                    "average_relevant_docs_per_query": 1.0005973715651135,
                },
                "vie-eng": {
                    "average_document_length": 913.8633993743483,
                    "average_query_length": 48.11086837793555,
                    "num_documents": 4795,
                    "num_queries": 5493,
                    "average_relevant_docs_per_query": 1.0003640997633352,
                },
                "vie-spa": {
                    "average_document_length": 940.0322580645161,
                    "average_query_length": 51.13386217154189,
                    "num_documents": 1829,
                    "num_queries": 2017,
                    "average_relevant_docs_per_query": 1.0004957858205255,
                },
                "vie-hin": {
                    "average_document_length": 838.1713414634146,
                    "average_query_length": 47.484334874165384,
                    "num_documents": 1640,
                    "num_queries": 1947,
                    "average_relevant_docs_per_query": 1.0,
                },
                "vie-vie": {
                    "average_document_length": 913.8633993743483,
                    "average_query_length": 48.094085532302095,
                    "num_documents": 4795,
                    "num_queries": 5495,
                    "average_relevant_docs_per_query": 1.0,
                },
                "vie-zho": {
                    "average_document_length": 999.064534883721,
                    "average_query_length": 15.045805455481215,
                    "num_documents": 1720,
                    "num_queries": 1943,
                    "average_relevant_docs_per_query": 1.0,
                },
                "zho-ara": {
                    "average_document_length": 253.71303841676368,
                    "average_query_length": 42.04866562009419,
                    "num_documents": 1718,
                    "num_queries": 1911,
                    "average_relevant_docs_per_query": 1.000523286237572,
                },
                "zho-deu": {
                    "average_document_length": 241.84631147540983,
                    "average_query_length": 52.25107958050586,
                    "num_documents": 1464,
                    "num_queries": 1621,
                    "average_relevant_docs_per_query": 1.0,
                },
                "zho-eng": {
                    "average_document_length": 247.55609326880776,
                    "average_query_length": 48.64167478091529,
                    "num_documents": 4546,
                    "num_queries": 5135,
                    "average_relevant_docs_per_query": 1.0003894839337877,
                },
                "zho-spa": {
                    "average_document_length": 254.44552196235026,
                    "average_query_length": 51.90446841294299,
                    "num_documents": 1753,
                    "num_queries": 1947,
                    "average_relevant_docs_per_query": 1.0,
                },
                "zho-hin": {
                    "average_document_length": 229.60590163934427,
                    "average_query_length": 49.06625141562854,
                    "num_documents": 1525,
                    "num_queries": 1766,
                    "average_relevant_docs_per_query": 1.0005662514156286,
                },
                "zho-vie": {
                    "average_document_length": 266.1140401146132,
                    "average_query_length": 49.27328872876994,
                    "num_documents": 1745,
                    "num_queries": 1943,
                    "average_relevant_docs_per_query": 1.0,
                },
                "zho-zho": {
                    "average_document_length": 247.55609326880776,
                    "average_query_length": 15.019080996884735,
                    "num_documents": 4546,
                    "num_queries": 5136,
                    "average_relevant_docs_per_query": 1.0001947040498442,
                },
            },
        },
    )

    def load_data(self, **kwargs):
        """In this retrieval datasets, corpus is in lang XX and queries in lang YY."""
        if self.data_loaded:
            return

        _dataset_raw = {}
        self.queries, self.corpus, self.relevant_docs = {}, {}, {}

        for hf_subset, langs in _LANGUAGES.items():
            # Builds a language pair separated by an underscore. e.g., "ara-Arab_eng-Latn".
            # Corpus is in ara-Arab and queries in eng-Latn
            lang_pair = _build_lang_pair(langs)

            _dataset_raw[lang_pair] = datasets.load_dataset(
                name=hf_subset,
                **self.metadata_dict["dataset"],
            )

            self.queries[lang_pair] = {}
            self.corpus[lang_pair] = {}
            self.relevant_docs[lang_pair] = {}

            for eval_split in self.metadata.eval_splits:
                self.queries[lang_pair][eval_split] = {}
                self.corpus[lang_pair][eval_split] = {}
                self.relevant_docs[lang_pair][eval_split] = {}

                split_data = _dataset_raw[lang_pair][eval_split]
                query_ids = {
                    query: f"Q{i}"
                    for i, query in enumerate(set(split_data["question"]))
                }
                context_ids = {
                    text: f"C{i}" for i, text in enumerate(set(split_data["context"]))
                }

                for row in split_data:
                    query = row["question"]
                    context = row["context"]
                    query_id = query_ids[query]
                    context_id = context_ids[context]
                    self.queries[lang_pair][eval_split][query_id] = query
                    self.corpus[lang_pair][eval_split][context_id] = {
                        "title": "",
                        "text": context,
                    }
                    if query_id not in self.relevant_docs[lang_pair][eval_split]:
                        self.relevant_docs[lang_pair][eval_split][query_id] = {}
                    self.relevant_docs[lang_pair][eval_split][query_id][context_id] = 1

        self.data_loaded = True



from __future__ import annotations

import datasets

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class SpanishPassageRetrievalS2S(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SpanishPassageRetrievalS2S",
        description="Test collection for passage retrieval from health-related Web resources in Spanish.",
        reference="https://mklab.iti.gr/results/spanish-passage-retrieval-dataset/",
        dataset={
            "path": "jinaai/spanish_passage_retrieval",
            "revision": "9cddf2ce5209ade52c2115ccfa00eb22c6d3a837",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2s",
        eval_splits=["test"],
        eval_langs=["spa-Latn"],
        main_score="ndcg_at_10",
        date=None,
        form=None,
        domains=None,
        task_subtypes=None,
        license=None,
        socioeconomic_status=None,
        annotations_creators=None,
        dialect=None,
        text_creation=None,
        bibtex_citation="""@InProceedings{10.1007/978-3-030-15719-7_19,
author="Kamateri, Eleni
and Tsikrika, Theodora
and Symeonidis, Spyridon
and Vrochidis, Stefanos
and Minker, Wolfgang
and Kompatsiaris, Yiannis",
editor="Azzopardi, Leif
and Stein, Benno
and Fuhr, Norbert
and Mayr, Philipp
and Hauff, Claudia
and Hiemstra, Djoerd",
title="A Test Collection for Passage Retrieval Evaluation of Spanish Health-Related Resources",
booktitle="Advances in Information Retrieval",
year="2019",
publisher="Springer International Publishing",
address="Cham",
pages="148--154",
abstract="This paper describes a new test collection for passage retrieval from health-related Web resources in Spanish. The test collection contains 10,037 health-related documents in Spanish, 37 topics representing complex information needs formulated in a total of 167 natural language questions, and manual relevance assessments of text passages, pooled from multiple systems. This test collection is the first to combine search in a language beyond English, passage retrieval, and health-related resources and topics targeting the general public.",
isbn="978-3-030-15719-7"
}
""",
        n_samples=None,
        avg_character_length={
            "test": {
                "average_document_length": 434.5924528301887,
                "average_query_length": 67.55688622754491,
                "num_documents": 265,
                "num_queries": 167,
                "average_relevant_docs_per_query": 7.718562874251497,
            }
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        query_rows = datasets.load_dataset(
            name="queries",
            split="test",
            **self.metadata_dict["dataset"],
        )
        corpus_rows = datasets.load_dataset(
            name="corpus.sentences",
            split="test",
            **self.metadata_dict["dataset"],
        )
        qrels_rows = datasets.load_dataset(
            name="qrels.s2s",
            split="test",
            **self.metadata_dict["dataset"],
        )

        self.queries = {"test": {row["_id"]: row["text"] for row in query_rows}}
        self.corpus = {"test": {row["_id"]: row for row in corpus_rows}}
        self.relevant_docs = {
            "test": {
                row["_id"]: {v: 1 for v in row["text"].split(" ")} for row in qrels_rows
            }
        }

        self.data_loaded = True



from mteb.abstasks import AbsTaskClassification  # type: ignore
from mteb.abstasks.TaskMetadata import TaskMetadata  # type: ignore


class Moroco(AbsTaskClassification):
    metadata = TaskMetadata(
        name="Moroco",
        dataset={
            "path": "universityofbucharest/moroco",
            "revision": "d64d9b8cd876056a5c24552afe3caf7e6fd26c8e",
            "trust_remote_code": True,
        },
        description="The Moldavian and Romanian Dialectal Corpus. The MOROCO data set contains Moldavian and Romanian samples of text collected from the news domain. The samples belong to one of the following six topics: (0) culture, (1) finance, (2) politics, (3) science, (4) sports, (5) tech",
        reference="https://huggingface.co/datasets/moroco",
        type="Classification",
        category="s2s",
        eval_splits=["test"],
        eval_langs=["ron-Latn"],
        main_score="accuracy",
        date=("2017-10-01", "2017-10-31"),
        form=["written"],
        domains=["News"],
        task_subtypes=["Topic classification"],
        license="CC BY-4.0",
        socioeconomic_status="medium",
        annotations_creators="derived",
        dialect=[
            "ron-Latn-ron",
            "ron-Latn-mol",
        ],  # Moldavian, or the Romanian dialect used in Moldova, does not have an ISO 639-1 code assigned to it. However, it has been given the three-letter code "mol" under ISO 639-3
        text_creation="found",
        bibtex_citation=""""
        @inproceedings{ Butnaru-ACL-2019,
        author = {Andrei M. Butnaru and Radu Tudor Ionescu},
        title = "{MOROCO: The Moldavian and Romanian Dialectal Corpus}",
        booktitle = {Proceedings of ACL},
        year = {2019},
        pages={688--698},
        }
        """,
        n_samples={"test": 2048},
        avg_character_length={"test": 1710.94},
    )

    def dataset_transform(self):
        self.dataset = self.dataset.rename_columns(
            {"sample": "text", "category": "label"}
        ).remove_columns(["id"])
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )



from __future__ import annotations

from typing import Any

import datasets

from mteb.abstasks.AbsTaskPairClassification import AbsTaskPairClassification
from mteb.abstasks.TaskMetadata import TaskMetadata

_DATASET_COLUMN_MAP = [
    {
        "name": "citation_prediction_classification",
        "sent1": "citation",
        "sent2": "text",
        "labels": "answer",
        "mapping": {"yes": 1, "no": 0},
    },
    {
        "name": "consumer_contracts_qa",
        "sent1": "question",
        "sent2": "contract",
        "labels": "answer",
        "mapping": {"yes": 1, "no": 0},
    },
    {
        "name": "contract_qa",
        "sent1": "question",
        "sent2": "text",
        "labels": "answer",
        "mapping": {"yes": 1, "no": 0},
    },
    {
        "name": "hearsay",
        "sent1": "text",
        "sent2": "slice",
        "labels": "answer",
        "mapping": {"yes": 1, "no": 0},
    },
    {
        "name": "privacy_policy_entailment",
        "sent1": "text",
        "sent2": "description",
        "labels": "answer",
        "mapping": {"correct": 1, "incorrect": 0},
    },
    {
        "name": "privacy_policy_qa",
        "sent1": "text",
        "sent2": "question",
        "labels": "answer",
        "mapping": {"relevant": 1, "irrelevant": 0},
    },
]


class LegalBenchPC(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="LegalBenchPC",
        description="""This LegalBench pair classification task is a combination of the following datasets:

        - Citation Prediction Classification: Given a legal statement and a case citation, determine if the citation is supportive of the legal statement.
        - Consumer Contracts QA: The task consists of 400 yes/no questions relating to consumer contracts (specifically, online terms of service) and is relevant to the legal skill of contract interpretation.
        - Contract QA: Answer yes/no questions about whether contractual clauses discuss particular issues like confidentiality requirements, BIPA consent, PII data breaches, breach of contract etc.
        - Hearsay: Classify if a particular piece of evidence qualifies as hearsay. Each sample in the dataset describes (1) an issue being litigated or an assertion a party wishes to prove, and (2) a piece of evidence a party wishes to introduce. The goal is to determine if—as it relates to the issue—the evidence would be considered hearsay under the definition provided above.
        - Privacy Policy Entailment: Given a privacy policy clause and a description of the clause, determine if the description is correct. This is a binary classification task in which the LLM is provided with a clause from a privacy policy, and a description of that clause (e.g., “The policy describes collection of the user’s HTTP cookies, flash cookies, pixel tags, or similar identifiers by a party to the contract.”).
        - Privacy Policy QA: Given a question and a clause from a privacy policy, determine if the clause contains enough information to answer the question. This is a binary classification task in which the LLM is provided with a question (e.g., “do you publish my data”) and a clause from a privacy policy. The LLM must determine if the clause contains an answer to the question, and classify the question-clause pair.
        """,
        reference="https://huggingface.co/datasets/nguha/legalbench",
        dataset={
            "path": "nguha/legalbench",
            "revision": "12ca3b695563788fead87a982ad1a068284413f4",
            "trust_remote_code": True,
        },
        type="PairClassification",
        category="s2s",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2000-01-01", "2023-08-23"),  # best guess
        form=["written"],
        domains=["Legal"],
        task_subtypes=[],
        license="cc-by-4.0",
        socioeconomic_status="high",
        annotations_creators="expert-annotated",
        dialect=[],
        text_creation="found",
        bibtex_citation="""
        @misc{guha2023legalbench,
            title={LegalBench: A Collaboratively Built Benchmark for Measuring Legal Reasoning in Large Language Models}, 
            author={Neel Guha and Julian Nyarko and Daniel E. Ho and Christopher Ré and Adam Chilton and Aditya Narayana and Alex Chohlas-Wood and Austin Peters and Brandon Waldon and Daniel N. Rockmore and Diego Zambrano and Dmitry Talisman and Enam Hoque and Faiz Surani and Frank Fagan and Galit Sarfaty and Gregory M. Dickinson and Haggai Porat and Jason Hegland and Jessica Wu and Joe Nudell and Joel Niklaus and John Nay and Jonathan H. Choi and Kevin Tobia and Margaret Hagan and Megan Ma and Michael Livermore and Nikon Rasumov-Rahe and Nils Holzenberger and Noam Kolt and Peter Henderson and Sean Rehaag and Sharad Goel and Shang Gao and Spencer Williams and Sunny Gandhi and Tom Zur and Varun Iyer and Zehua Li},
            year={2023},
            eprint={2308.11462},
            archivePrefix={arXiv},
            primaryClass={cs.CL}
            }
        @article{kolt2022predicting,
            title={Predicting consumer contracts},
            author={Kolt, Noam},
            journal={Berkeley Tech. LJ},
            volume={37},
            pages={71},
            year={2022},
            publisher={HeinOnline}
        }
        @article{zimmeck2019maps,
            title={Maps: Scaling privacy compliance analysis to a million apps},
            author={Zimmeck, Sebastian and Story, Peter and Smullen, Daniel and Ravichander, Abhilasha and Wang, Ziqi and Reidenberg, Joel R and Russell, N Cameron and Sadeh, Norman},
            journal={Proc. Priv. Enhancing Tech.},
            volume={2019},
            pages={66},
            year={2019}
        }
        @article{ravichander2019question,
            title={Question answering for privacy policies: Combining computational and legal perspectives},
            author={Ravichander, Abhilasha and Black, Alan W and Wilson, Shomir and Norton, Thomas and Sadeh, Norman},
            journal={arXiv preprint arXiv:1911.00841},
            year={2019}
        }
        """,
        n_samples={"test": 2048},
        avg_character_length={"test": 287.18},
    )

    def load_data(self, **kwargs: Any) -> None:
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return

        _hf_dataset = None
        for dataset_col_map in _DATASET_COLUMN_MAP:
            _dataset = datasets.load_dataset(
                self.metadata_dict["dataset"]["path"],
                dataset_col_map["name"],
                revision=self.metadata_dict["dataset"]["revision"],
                trust_remote_code=True,
            )

            _dataset = _dataset.rename_columns(
                {
                    dataset_col_map["sent1"]: "sentence1",
                    dataset_col_map["sent2"]: "sentence2",
                    dataset_col_map["labels"]: "labels",
                }
            )
            _dataset = _dataset.select_columns(["labels", "sentence1", "sentence2"])
            mapping = dataset_col_map["mapping"]
            _dataset = _dataset.map(
                lambda example: {
                    "labels": mapping.get(example["labels"].lower(), example["labels"])
                }
            )

            if _hf_dataset is None:
                _hf_dataset = _dataset
            else:
                _hf_dataset["train"] = datasets.concatenate_datasets(
                    [_hf_dataset["train"], _dataset["train"]]
                )
                _hf_dataset["test"] = datasets.concatenate_datasets(
                    [_hf_dataset["test"], _dataset["test"]]
                )

        self.dataset = _hf_dataset
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"], label="labels"
        )

        _dataset = {}
        for split in self.metadata.eval_splits:
            hf_dataset = self.dataset[split]
            _dataset[split] = [
                {
                    "sentence1": hf_dataset["sentence1"],
                    "sentence2": hf_dataset["sentence2"],
                    "labels": hf_dataset["labels"],
                }
            ]
        self.dataset = _dataset



from __future__ import annotations

import logging

from datasets import Dataset

from ..evaluation.evaluators import BitextMiningEvaluator
from ..MTEBResults import HFSubset, ScoresDict
from .AbsTask import AbsTask

logger = logging.getLogger(__name__)


class AbsTaskBitextMining(AbsTask):
    """Abstract class for BitextMining tasks
    The similarity is computed between pairs and the results are ranked.

    self.load_data() must generate a huggingface dataset with a split matching self.metadata_dict["eval_splits"], and assign it to self.dataset. It must contain the following columns:
        id: str
        sentence1: str
        sentence2: str
    """

    parallel_subsets = False

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def evaluate(self, model, split, **kwargs) -> dict[HFSubset, ScoresDict]:
        if not self.data_loaded:
            self.load_data()

        hf_subsets = [l for l in self.dataset] if self.is_multilingual else ["default"]

        scores = {}
        if self.parallel_subsets:
            scores = self._evaluate_subset(
                model,
                self.dataset[split],  # type: ignore
                parallel=True,
                **kwargs,
            )
        else:
            for hf_subet in hf_subsets:
                logger.info(
                    f"\nTask: {self.metadata_dict['name']}, split: {split}, subset: {hf_subet}. Running..."
                )

                if hf_subet not in self.dataset and hf_subet == "default":
                    data_split = self.dataset[split]
                else:
                    data_split = self.dataset[hf_subet][split]
                scores[hf_subet] = self._evaluate_subset(
                    model,
                    data_split,  # type: ignore
                    subsets=["sentence1", "sentence2"],
                    **kwargs,
                )

        return scores

    def _evaluate_subset(
        self, model, data_split: Dataset, parallel=False, **kwargs
    ) -> ScoresDict:
        evaluator = BitextMiningEvaluator(
            data_split, task_name=self.metadata.name, **kwargs
        )
        metrics = evaluator(model)
        if parallel:
            for v in metrics.values():
                self._add_main_score(v)
        else:
            self._add_main_score(metrics)
        return metrics

    def _add_main_score(self, scores) -> None:
        scores["main_score"] = scores[self.metadata.main_score]



"""This specifies the default instructions for tasks within MTEB. These are optional to use and some models might want to use their own instructions."""

import mteb

# Prompts from
# SEB: https://github.com/KennethEnevoldsen/scandinavian-embedding-benchmark/blob/c8376f967d1294419be1d3eb41217d04cd3a65d3/src/seb/registered_models/e5_instruct_models.py
# E5: https://github.com/microsoft/unilm/blob/9c0f1ff7ca53431fe47d2637dfe253643d94185b/e5/utils.py#L106
DEFAULT_PROMPTS = {
    "STS": "Retrieve semantically similar text.",
    "Summarization": "Given a news summary, retrieve other semantically similar summaries",
    "BitextMining": "Retrieve parallel sentences.",
    "Classification": "Classify user passages",
    "Clustering": "Identify categories in user passages",
    "Reranking": "Retrieve text based on user query.",
    "Retrieval": "Retrieve text based on user query.",
    "InstructionRetrieval": "Retrieve text based on user query.",
    "PairClassification": "Retrieve text that are semantically similar to the given text",
}


# This list is NOT comprehensive even for the tasks within MTEB
# TODO: We should probably move this prompt to the task object
TASKNAME2INSTRUCTIONS = {
    # BitextMining
    "BornholmBitextMining": "Retrieve parallel sentences in Danish and Bornholmsk",
    "NorwegianCourtsBitextMining ": "Retrieve parallel sentences in Norwegian Bokmål and Nynorsk",
    # Classification
    "AngryTweetsClassification": "Classify Danish tweets by sentiment. (positive, negative, neutral)",
    "DKHateClassification": "Classify Danish tweets based on offensiveness (offensive, not offensive)",
    "DanishPoliticalCommentsClassification": "Classify Danish political comments for sentiment",
    "DalajClassification": "Classify texts based on linguistic acceptability in Swedish",
    "LccSentimentClassification": "Classify texts based on sentiment",
    "NordicLangClassification": "Classify texts based on language",
    "MassiveIntentClassification": "Given a user utterance as query, find the user intents",
    "Massive Scenario": "Given a user utterance as query, find the user scenarios",
    "NoRecClassification": "Classify Norwegian reviews by sentiment",
    "SweRecClassification": "Classify Swedish reviews by sentiment",
    "Norwegian parliament": "Classify parliament speeches in Norwegian based on political affiliation",
    "ScalaClassification": "Classify passages in Scandinavian Languages based on linguistic acceptability",
    "AmazonCounterfactualClassification": "Classify a given Amazon customer review text as either counterfactual or not-counterfactual",
    "AmazonPolarityClassification": "Classify Amazon reviews into positive or negative sentiment",
    "AmazonReviewsClassification": "Classify the given Amazon review into its appropriate rating category",
    "Banking77Classification": "Given a online banking query, find the corresponding intents",
    "EmotionClassification": "Classify the emotion expressed in the given Twitter message into one of the six emotions: anger, fear, joy, love, sadness, and surprise",
    "ImdbClassification": "Classify the sentiment expressed in the given movie review text from the IMDB dataset",
    "MassiveScenarioClassification": "Given a user utterance as query, find the user scenarios",
    "MTOPDomainClassification": "Classify the intent domain of the given utterance in task-oriented conversation",
    "MTOPIntentClassification": "Classify the intent of the given utterance in task-oriented conversation",
    "ToxicConversationsClassification": "Classify the given comments as either toxic or not toxic",
    "TweetSentimentExtractionClassification": "Classify the sentiment of a given tweet as either positive, negative, or neutral",
    "TNews": "Classify the fine-grained category of the given news title",
    "IFlyTek": "Given an App description text, find the appropriate fine-grained category",
    "MultilingualSentiment": "Classify sentiment of the customer review into positive, neutral, or negative",
    "JDReview": "Classify the customer review for iPhone on e-commerce platform into positive or negative",
    "OnlineShopping": "Classify the customer review for online shopping into positive or negative",
    "Waimai": "Classify the customer review from a food takeaway platform into positive or negative",
    # Clustering
    "VGHierarchicalClusteringP2P": "Identify the categories (e.g. sports) of given articles in Norwegian",
    "VGHierarchicalClusteringS2S": "Identify the categories (e.g. sports) of given articles in Norwegian",
    "SNLHierarchicalClusteringP2P": "Identify categories in a Norwegian lexicon",
    "SNLHierarchicalClusteringS2S": "Identify categories in a Norwegian lexicon",
    "SwednClusteringP2P": "Identify news categories in Swedish passages",
    "SwednClusteringS2S": "Identify news categories in Swedish passages",
    "ArxivClusteringP2P": "Identify the main and secondary category of Arxiv papers based on the titles and abstracts",
    "ArxivClusteringS2S": "Identify the main and secondary category of Arxiv papers based on the titles",
    "BiorxivClusteringP2P": "Identify the main category of Biorxiv papers based on the titles and abstracts",
    "BiorxivClusteringS2S": "Identify the main category of Biorxiv papers based on the titles",
    "MedrxivClusteringP2P": "Identify the main category of Medrxiv papers based on the titles and abstracts",
    "MedrxivClusteringS2S": "Identify the main category of Medrxiv papers based on the titles",
    "RedditClustering": "Identify the topic or theme of Reddit posts based on the titles",
    "RedditClusteringP2P": "Identify the topic or theme of Reddit posts based on the titles and posts",
    "StackExchangeClustering": "Identify the topic or theme of StackExchange posts based on the titles",
    "StackExchangeClusteringP2P": "Identify the topic or theme of StackExchange posts based on the given paragraphs",
    "TwentyNewsgroupsClustering": "Identify the topic or theme of the given news articles",
    "CLSClusteringS2S": "Identify the main category of scholar papers based on the titles",
    "CLSClusteringP2P": "Identify the main category of scholar papers based on the titles and abstracts",
    "ThuNewsClusteringS2S": "Identify the topic or theme of the given news articles based on the titles",
    "ThuNewsClusteringP2P": "Identify the topic or theme of the given news articles based on the titles and contents",
    # Reranking and pair classification
    "AskUbuntuDupQuestions": "Retrieve duplicate questions from AskUbuntu forum",
    "MindSmallReranking": "Retrieve relevant news articles based on user browsing history",
    "SciDocsRR": "Given a title of a scientific paper, retrieve the titles of other relevant papers",
    "StackOverflowDupQuestions": "Retrieve duplicate questions from StackOverflow forum",
    "SprintDuplicateQuestions": "Retrieve duplicate questions from Sprint forum",
    "TwitterSemEval2015": "Retrieve tweets that are semantically similar to the given tweet",
    "TwitterURLCorpus": "Retrieve tweets that are semantically similar to the given tweet",
    "T2Reranking": "Given a Chinese search query, retrieve web passages that answer the question",
    "MMarcoReranking": "Given a Chinese search query, retrieve web passages that answer the question",
    "VoyageMMarcoReranking": "Given a Japanese search query, retrieve web passages that answer the question",
    "CMedQAv1": "Given a Chinese community medical question, retrieve replies that best answer the question",
    "CMedQAv2": "Given a Chinese community medical question, retrieve replies that best answer the question",
    "Ocnli": "Retrieve semantically similar text.",
    "Cmnli": "Retrieve semantically similar text.",
    # Retrieval - 1st item is query instruction; 2nd is corpus instruction
    "TwitterHjerneRetrieval": (
        "Retrieve answers to questions asked in Danish tweets",
        "",
    ),
    "SwednRetrieval": (
        "Given a Swedish news headline retrieve summaries or news articles",
        "",
    ),
    "TV2Nordretrieval": (
        "Given a summary of a Danish news article retrieve the corresponding news article",
        "",
    ),
    "DanFEVER": (
        "Given a claim in Danish, retrieve documents that support the claim",
        "",
    ),
    "SNLRetrieval": ("Given a lexicon headline in Norwegian, retrieve its article", ""),
    "NorQuadRetrieval": (
        "Given a question in Norwegian, retrieve the answer from Wikipedia articles",
        "",
    ),
    "SweFaqRetrieval": ("Retrieve answers given questions in Swedish", ""),
    "ArguAna": ("Given a claim, find documents that refute the claim", ""),
    "ClimateFEVER": (
        "Given a claim about climate change, retrieve documents that support or refute the claim",
        "",
    ),
    "DBPedia": (
        "Given a query, retrieve relevant entity descriptions from DBPedia",
        "",
    ),
    "FEVER": ("Given a claim, retrieve documents that support or refute the claim", ""),
    "FiQA2018": (
        "Given a financial question, retrieve user replies that best answer the question",
        "",
    ),
    "HotpotQA": (
        "Given a multi-hop question, retrieve documents that can help answer the question",
        "",
    ),
    "MSMARCO": (
        "Given a web search query, retrieve relevant passages that answer the query",
        "",
    ),
    "NFCorpus": (
        "Given a question, retrieve relevant documents that best answer the question",
        "",
    ),
    "NQ": (
        "Given a question, retrieve Wikipedia passages that answer the question",
        "",
    ),
    "QuoraRetrieval": (
        "Given a question, retrieve questions that are semantically equivalent to the given question",
        "",
    ),
    "SCIDOCS": (
        "Given a scientific paper title, retrieve paper abstracts that are cited by the given paper",
        "",
    ),
    "SciFact": (
        "Given a scientific claim, retrieve documents that support or refute the claim",
        "",
    ),
    "Touche2020": (
        "Given a question, retrieve detailed and persuasive arguments that answer the question",
        "",
    ),
    "TRECCOVID": (
        "Given a query on COVID-19, retrieve documents that answer the query",
        "",
    ),
    "T2Retrieval": (
        "Given a Chinese search query, retrieve web passages that answer the question",
        "",
    ),
    "MMarcoRetrieval": (
        "Given a web search query, retrieve relevant passages that answer the query",
        "",
    ),
    "DuRetrieval": (
        "Given a Chinese search query, retrieve web passages that answer the question",
        "",
    ),
    "CovidRetrieval": (
        "Given a question on COVID-19, retrieve news articles that answer the question",
        "",
    ),
    "CmedqaRetrieval": (
        "Given a Chinese community medical question, retrieve replies that best answer the question",
        "",
    ),
    "EcomRetrieval": (
        "Given a user query from an e-commerce website, retrieve description sentences of relevant products",
        "",
    ),
    "MedicalRetrieval": (
        "Given a medical question, retrieve user replies that best answer the question",
        "",
    ),
    "VideoRetrieval": (
        "Given a video search query, retrieve the titles of relevant videos",
        "",
    ),
}


def task_to_instruction(task_name: str, is_query: bool = True) -> str:
    if task_name in TASKNAME2INSTRUCTIONS:
        if isinstance(TASKNAME2INSTRUCTIONS[task_name], tuple):
            return (
                TASKNAME2INSTRUCTIONS[task_name][0]
                if is_query
                else TASKNAME2INSTRUCTIONS[task_name][1]
            )
        return TASKNAME2INSTRUCTIONS[task_name]

    meta = mteb.get_task(task_name).metadata
    return DEFAULT_PROMPTS.get(meta.type, "")



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class ItalianLinguisticAcceptabilityClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="Itacola",
        dataset={
            "path": "gsarti/itacola",
            "revision": "f8f98e5c4d3059cf1a00c8eb3d70aa271423f636",
            "trust_remote_code": True,
        },
        description="An Italian Corpus of Linguistic Acceptability taken from linguistic literature with a binary annotation made by the original authors themselves.",
        reference="https://aclanthology.org/2021.findings-emnlp.250/",
        type="Classification",
        category="s2s",
        eval_splits=["test"],
        eval_langs=["ita-Latn"],
        main_score="accuracy",
        date=("2021-01-01", "2021-12-31"),
        form=["written"],
        domains=["Non-fiction", "Spoken"],
        dialect=[],
        task_subtypes=["Linguistic acceptability"],
        license="unknown",
        socioeconomic_status="high",
        annotations_creators="expert-annotated",
        text_creation="found",
        bibtex_citation="""
        @inproceedings{trotta-etal-2021-monolingual-cross,
    title = "Monolingual and Cross-Lingual Acceptability Judgments with the {I}talian {C}o{LA} corpus",
    author = "Trotta, Daniela  and
      Guarasci, Raffaele  and
      Leonardelli, Elisa  and
      Tonelli, Sara",
    booktitle = "Findings of the Association for Computational Linguistics: EMNLP 2021",
    month = nov,
    year = "2021",
    address = "Punta Cana, Dominican Republic",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.findings-emnlp.250",
    doi = "10.18653/v1/2021.findings-emnlp.250",
    pages = "2929--2940"
}
        """,
        n_samples={"train": 7801, "test": 975},
        avg_character_length={"train": 35.95, "test": 36.67},
    )

    def dataset_transform(self):
        self.dataset = (
            self.dataset.rename_columns({"acceptability": "label"})
            .rename_columns({"sentence": "text"})
            .remove_columns(["unique_id", "source"])
        )



from __future__ import annotations

import os

from mteb import MTEB

HEADER = "| Name | Hub URL | Description | Type | Category | #Languages | Train #Samples | Dev #Samples | Test #Samples | Avg. chars / train | Avg. chars / dev | Avg. chars / test"
SEP = "|:-----|:-----|:-----|:-----|:-----|-----:|-----:|-----:|-----:|-----:|-----:|-----:|"
ONE_LINE = "| {} | {} | {} | {} | {} | {} | {} | {} | {} | {} | {} | {} |"

TABLE_STRING = "\n".join([HEADER, SEP])

LEN_KEYS = {
    "text",
    "sentences",
    "sentence1",
    "sentence2",
    "sent1",
    "sent2" "query",
    "positive",
    "negative" "queries",
    "corpus",
    "machine_summaries",
    "human_summaries",
}


DATAPATH = "/gpfsscratch/rech/six/commun/commun/experiments/muennighoff/mteb"


def load_data(hf_hub_name, subset=None):
    """Load dataset from Hub via cloning for easy offline usage with HF_DATASETS_OFFLINE=1
    Can be replaced with just `load_dataset(hf_hub_name, subset)` if preferred
    """
    from datasets import load_dataset

    path = os.path.join(DATAPATH, hf_hub_name)
    if os.path.exists(path):
        dataset = load_dataset(path, subset)
    else:
        from git import Repo

        Repo.clone_from("https://huggingface.co/datasets/" + hf_hub_name, path)
        dataset = load_dataset(path, subset)
    return dataset


def get_ds_stats_beir_hub(hf_hub_name):
    """Not used as some BEIR datasets are still missing on the Hub"""
    lens = {}
    for subset in ["corpus", "queries"]:
        ds = load_data("mteb/hfbeir" + hf_hub_name.replace("BeIR", ""), subset)
        splits = list(ds.keys())
        len_keys = set(ds[splits[-1]].features.keys()) & LEN_KEYS
        for split in splits:
            if split not in ds:
                continue
            lens.setdefault(split, [])
            for k in len_keys:
                if isinstance(ds[split][k][0], str):
                    lens[split] += [len(x) for x in ds[split][k]]
                elif isinstance(ds[split][k][0], list):
                    assert isinstance(ds[split][k][0][0], str), f"Too nested: {k}"
                    lens[split] += [len(y) for x in ds[split][k] for y in x]
                else:
                    raise ValueError(f"Unknown type {type(ds[split][k])}")
    all_lens = [x for y in lens.values() for x in y]
    avg_len = sum(all_lens) / len(all_lens)
    return ["TODO"] * 3 + [round(avg_len, 1)] * 3


def get_ds_stats_beir(hf_hub_name):
    from beir.datasets.data_loader import GenericDataLoader as BeirDataLoader

    path = os.path.join(DATAPATH, hf_hub_name)
    if not os.path.exists(path):
        from beir import util

        url = f"https://public.ukp.informatik.tu-darmstadt.de/thakur/BEIR/datasets/{hf_hub_name}.zip"
        util.download_and_unzip(url, DATAPATH)
    lens = {"train": [], "dev": [], "test": []}
    for split in lens.keys():
        try:
            corpus, queries, relevant_docs = BeirDataLoader(path).load(split=split)
        except:  # split does not exist  # noqa: E722
            continue
        # + 1 for space added between Title & Text by default in BEIR
        avg_lens_c = [len(v["text"]) + len(v["title"]) + 1 for v in corpus.values()]
        avg_lens_q = [len(v) for v in queries.values()]
        lens[split].extend(avg_lens_c)
        lens[split].extend(avg_lens_q)
    avg_lens = {
        k: round(sum(lens[k]) / len(lens[k]), 1) if lens[k] else 0 for k in lens
    }
    return (
        len(lens["train"]),
        len(lens["dev"]),
        len(lens["test"]),
        avg_lens["train"],
        avg_lens["dev"],
        avg_lens["test"],
    )


def get_ds_stats(hf_hub_name):
    ds = load_data(hf_hub_name)
    assert "test" in ds, f"No test set for {hf_hub_name}"
    len_keys = set(ds["test"].features.keys()) & LEN_KEYS
    dev_key = "dev" if "dev" in ds else "validation"
    lens = {"train": [], dev_key: [], "test": []}

    for split in lens.keys():
        if split not in ds:
            continue
        for k in len_keys:
            if isinstance(ds[split][k][0], str):
                lens[split] += [len(x) for x in ds[split][k]]
            elif isinstance(ds[split][k][0], list):
                assert isinstance(ds[split][k][0][0], str), f"Too nested: {k}"
                lens[split] += [len(y) for x in ds[split][k] for y in x]
            else:
                raise ValueError(f"Unknown type {type(ds[split][k])}")

    avg_lens = {
        k: round(sum(lens[k]) / len(lens[k]), 1) if lens[k] else 0 for k in lens
    }
    return (
        len(lens["train"]),
        len(lens[dev_key]),
        len(lens["test"]),
        avg_lens["train"],
        avg_lens[dev_key],
        avg_lens["test"],
    )


# Select all tasks
for task in MTEB().tasks:
    print("Task: ", task)
    if "dataset" in task.metadata_dict:
        hub_name = hub_url = task.metadata_dict["dataset"]["path"]
        ds_stats = get_ds_stats(hub_name.split("/")[-1])
    elif "beir_name" in task.metadata_dict:
        hub_name = hub_url = "BeIR/" + task.metadata_dict.get("beir_name")
        ds_stats = get_ds_stats_beir("/".join(hub_name.split("/")[1:]))
        if "cqadupstack" in hub_name:
            hub_url = "BeIR/cqadupstack-qrels"
    TABLE_STRING += "\n" + ONE_LINE.format(
        f"[{task.metadata_dict['name']}]({task.metadata_dict['reference']})",
        f"[{hub_name}](https://huggingface.co/datasets/{hub_url})",
        task.metadata_dict["description"],
        task.metadata_dict["type"],
        task.metadata_dict["category"],
        len(task.metadata_dict["eval_langs"]),
        *ds_stats,
    )

with open("./mdtable.md", "w") as f:
    f.write(TABLE_STRING)

# Convert to latex
for line in TABLE_STRING.split("\n")[2:]:
    if line:
        cols = line.split(" | ")
        idx = cols[0].index("]")
        cols[0] = cols[0][3:idx]
        cols[-1] = cols[-1][:-1]
        out = " & ".join(cols[:1] + cols[3:]) + " \\\\"
        print(out)



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class Touche2020(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="Touche2020",
        description="Touché Task 1: Argument Retrieval for Controversial Questions",
        reference="https://webis.de/events/touche-20/shared-task-1.html",
        dataset={
            "path": "mteb/touche2020",
            "revision": "a34f9a33db75fa0cbb21bb5cfc3dae8dc8bec93f",
        },
        type="Retrieval",
        category="s2p",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=None,
        form=None,
        domains=None,
        task_subtypes=None,
        license=None,
        socioeconomic_status=None,
        annotations_creators=None,
        dialect=None,
        text_creation=None,
        bibtex_citation="""@dataset{potthast_2022_6862281,
  author       = {Potthast, Martin and
                  Gienapp, Lukas and
                  Wachsmuth, Henning and
                  Hagen, Matthias and
                  Fröbe, Maik and
                  Bondarenko, Alexander and
                  Ajjour, Yamen and
                  Stein, Benno},
  title        = {{Touché20-Argument-Retrieval-for-Controversial- 
                   Questions}},
  month        = jul,
  year         = 2022,
  publisher    = {Zenodo},
  doi          = {10.5281/zenodo.6862281},
  url          = {https://doi.org/10.5281/zenodo.6862281}
}""",
        n_samples=None,
        avg_character_length={
            "test": {
                "average_document_length": 1719.3347658445412,
                "average_query_length": 43.42857142857143,
                "num_documents": 382545,
                "num_queries": 49,
                "average_relevant_docs_per_query": 19.020408163265305,
            }
        },
    )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class GerDaLIRSmall(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="GerDaLIRSmall",
        description="The dataset consists of documents, passages and relevance labels in German. In contrast to the original dataset, only documents that have corresponding queries in the query set are chosen to create a smaller corpus for evaluation purposes.",
        reference="https://github.com/lavis-nlp/GerDaLIR",
        dataset={
            "path": "mteb/GerDaLIRSmall",
            "revision": "48327de6ee192e9610f3069789719788957c7abd",
        },
        type="Retrieval",
        category="p2p",
        eval_splits=["test"],
        eval_langs=["deu-Latn"],
        main_score="ndcg_at_10",
        date=None,
        form=["written"],
        domains=["Legal"],
        task_subtypes=["Article retrieval"],
        license="MIT license",
        socioeconomic_status="high",
        annotations_creators="derived",
        dialect=None,
        text_creation="found",
        bibtex_citation="""@inproceedings{wrzalik-krechel-2021-gerdalir,
    title = "{G}er{D}a{LIR}: A {G}erman Dataset for Legal Information Retrieval",
    author = "Wrzalik, Marco  and
      Krechel, Dirk",
    booktitle = "Proceedings of the Natural Legal Language Processing Workshop 2021",
    month = nov,
    year = "2021",
    address = "Punta Cana, Dominican Republic",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.nllp-1.13",
    pages = "123--128",
    abstract = "We present GerDaLIR, a German Dataset for Legal Information Retrieval based on case documents from the open legal information platform Open Legal Data. The dataset consists of 123K queries, each labelled with at least one relevant document in a collection of 131K case documents. We conduct several baseline experiments including BM25 and a state-of-the-art neural re-ranker. With our dataset, we aim to provide a standardized benchmark for German LIR and promote open research in this area. Beyond that, our dataset comprises sufficient training data to be used as a downstream task for German or multilingual language models.",
}""",
        n_samples=None,
        avg_character_length={
            "test": {
                "average_document_length": 19706.823653325308,
                "average_query_length": 1031.0680889324833,
                "num_documents": 9969,
                "num_queries": 12234,
                "average_relevant_docs_per_query": 1.1705084191597188,
            }
        },
    )



from __future__ import annotations

import datasets

from mteb.abstasks import MultilingualTask
from mteb.abstasks.AbsTaskPairClassification import AbsTaskPairClassification
from mteb.abstasks.TaskMetadata import TaskMetadata

_LANGS = {
    "de": ["deu-Latn"],
    "en": ["eng-Latn"],
    "fr": ["fra-Latn"],
    "it": ["ita-Latn"],
}


class RTE3(MultilingualTask, AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="RTE3",
        dataset={
            "path": "maximoss/rte3-multi",
            "revision": "d94f96ca5a6798e20f5a77e566f7a288dc6138d7",
        },
        description="Recognising Textual Entailment Challenge (RTE-3) aim to provide the NLP community with a benchmark to test progress in recognizing textual entailment",
        reference="https://aclanthology.org/W07-1401/",
        category="s2s",
        type="PairClassification",
        eval_splits=["test"],
        eval_langs=_LANGS,
        main_score="ap",
        date=("2023-03-25", "2024-04-15"),
        form=["written"],
        domains=["News", "Web", "Encyclopaedic"],
        task_subtypes=["Textual Entailment"],
        license="cc-by-4.0",
        socioeconomic_status="mixed",
        annotations_creators="expert-annotated",
        dialect=[],
        text_creation="found",
        bibtex_citation="""@inproceedings{giampiccolo-etal-2007-third,
            title = "The Third {PASCAL} Recognizing Textual Entailment Challenge",
            author = "Giampiccolo, Danilo  and
            Magnini, Bernardo  and
            Dagan, Ido  and
            Dolan, Bill",
            booktitle = "Proceedings of the {ACL}-{PASCAL} Workshop on Textual Entailment and Paraphrasing",
            month = jun,
            year = "2007",
            address = "Prague",
            publisher = "Association for Computational Linguistics",
            url = "https://aclanthology.org/W07-1401",
            pages = "1--9",
        }
        """,
        n_samples={"test": 1923},  # sum of 4 languages after neutral filtering
        avg_character_length={"test": 124.79},
    )

    def load_data(self, **kwargs):
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.dataset = datasets.load_dataset(
            self.metadata.dataset["path"], revision=self.metadata.dataset["revision"]
        )
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self):
        _dataset = {}
        for lang in self.hf_subsets:
            _dataset[lang] = {}
            for split in self.metadata.eval_splits:
                # keep target language
                hf_dataset = self.dataset[split].filter(lambda x: x["language"] == lang)
                # keep labels 0=entailment and 2=contradiction, and map them as 1 and 0 for binary classification
                hf_dataset = hf_dataset.filter(lambda x: x["label"] in [0, 2])
                hf_dataset = hf_dataset.map(
                    lambda example: {"label": 0 if example["label"] == 2 else 1}
                )
                _dataset[lang][split] = [
                    {
                        "sentence1": hf_dataset["premise"],
                        "sentence2": hf_dataset["hypothesis"],
                        "labels": hf_dataset["label"],
                    }
                ]
        self.dataset = _dataset



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class RiaNewsRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="RiaNewsRetrieval",
        dataset={
            "path": "ai-forever/ria-news-retrieval",
            "revision": "82374b0bbacda6114f39ff9c5b925fa1512ca5d7",
        },
        description="News article retrieval by headline. Based on Rossiya Segodnya dataset.",
        reference="https://arxiv.org/abs/1901.07786",
        type="Retrieval",
        category="s2p",
        eval_splits=["test"],
        eval_langs=["rus-Cyrl"],
        main_score="ndcg_at_10",
        date=("2010-01-01", "2014-12-31"),
        form=["written"],
        domains=["News"],
        task_subtypes=["Article retrieval"],
        license="cc-by-nc-nd-4.0",
        socioeconomic_status="mixed",
        annotations_creators="derived",
        dialect=[],
        text_creation="found",
        bibtex_citation="""@inproceedings{gavrilov2018self,
        title={Self-Attentive Model for Headline Generation},
        author={Gavrilov, Daniil and  Kalaidin, Pavel and  Malykh, Valentin},
        booktitle={Proceedings of the 41st European Conference on Information Retrieval},
        year={2019}
        }""",
        n_samples={"test": 10000},
        avg_character_length={
            "test": {
                "average_document_length": 1165.6429557148213,
                "average_query_length": 62.4029,
                "num_documents": 704344,
                "num_queries": 10000,
                "average_relevant_docs_per_query": 1.0,
            }
        },
    )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class FeedbackQARetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="FeedbackQARetrieval",
        description="Using Interactive Feedback to Improve the Accuracy and Explainability of Question Answering Systems Post-Deployment",
        reference="https://arxiv.org/abs/2204.03025",
        dataset={
            "path": "lt2c/fqa",
            "revision": "1ee1cd0",
        },
        type="Retrieval",
        category="s2p",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="precision_at_1",
        date=("2020-01-01", "2022-04-01"),
        form=["written"],
        domains=["Web", "Government", "Medical"],
        task_subtypes=["Question answering"],
        license="Apache-2.0",
        socioeconomic_status="mixed",
        annotations_creators="human-annotated",
        dialect=[],
        text_creation="created",
        bibtex_citation="""
@inproceedings{li-etal-2022-using,
    title = "Using Interactive Feedback to Improve the Accuracy and Explainability of Question Answering Systems Post-Deployment",
    author = "Li, Zichao  and
      Sharma, Prakhar  and
      Lu, Xing Han  and
      Cheung, Jackie  and
      Reddy, Siva",
    editor = "Muresan, Smaranda  and
      Nakov, Preslav  and
      Villavicencio, Aline",
    booktitle = "Findings of the Association for Computational Linguistics: ACL 2022",
    month = may,
    year = "2022",
    address = "Dublin, Ireland",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2022.findings-acl.75",
    doi = "10.18653/v1/2022.findings-acl.75",
    pages = "926--937"
}
""",
        n_samples={"test": 1992},
        avg_character_length={
            "test": {
                "average_document_length": 1174.7986463620982,
                "average_query_length": 72.33182730923694,
                "num_documents": 2364,
                "num_queries": 1992,
                "average_relevant_docs_per_query": 1.0,
            }
        },
    )



"""Command line interface for various MTEB.

MTEB is a benchmark for evaluating the quality of embeddings in various tasks. It supports the following commands:

- mteb run: Runs a model on a set of tasks
- mteb available_tasks: Lists the available tasks within MTEB
- mteb create_meta: Creates the metadata for a model card from a folder of results

## Running Models on Tasks

To run a model on a set of tasks, use the `mteb run` command. For example:
    
```bash
mteb run -m average_word_embeddings_komninos \
         -t Banking77Classification EmotionClassification \
         --output_folder mteb_output \
          --verbosity 3
```

This will create a folder `mteb_output/{model_name}/{model_revision}` containing the results of the model on the specified tasks supplied as a json
file: "{task_name}.json".


## Listing Available Tasks

To list the available tasks within MTEB, use the `mteb available_tasks` command. For example:

```bash
mteb available_tasks # list all available tasks
mteb available_tasks --task_types Clustering # list tasks of type Clustering
```


## Creating Model Metadata

Once a model is run you can create the metadata for a model card from a folder of results, use the `mteb create_meta` command. For example:

```bash
mteb create_meta --results_folder mteb_output/average_word_embeddings_komninos/{revision} \
                 --output_path model_card.md
```

This will create a model card at `model_card.md` containing the metadata for the model on MTEB within the YAML frontmatter. This will make the model
discoverable on the MTEB leaderboard. 

An example frontmatter for a model card is shown below:

```
---
tags:
- mteb
model-index:
- name: SGPT-5.8B-weightedmean-msmarco-specb-bitfit
  results:
  - task:
      type: classification
    dataset:
      type: mteb/banking77
      name: MTEB Banking77
      config: default
      split: test
      revision: 44fa15921b4c889113cc5df03dd4901b49161ab7
    metrics:
    - type: accuracy
      value: 84.49350649350649
---
```
"""

import argparse
import json
import logging
from pathlib import Path

import yaml

import mteb
from mteb.MTEBResults import MTEBResults

logging.basicConfig(level=logging.WARNING)
logger = logging.getLogger(__name__)


def _save_model_metadata(model: mteb.Encoder, output_folder: Path) -> None:
    meta = model.mteb_model_meta  # type: ignore

    revision = meta.revision if meta.revision is not None else "no_revision_available"

    save_path = output_folder / meta.model_name_as_path() / revision / "model_meta.json"

    with save_path.open("w") as f:
        json.dump(meta.to_dict(), f)


def run(args: argparse.Namespace) -> None:
    # set logging based on verbosity level
    if args.verbosity == 0:
        logging.getLogger("mteb").setLevel(logging.CRITICAL)
    elif args.verbosity == 1:
        logging.getLogger("mteb").setLevel(logging.WARNING)
    elif args.verbosity == 2:
        logging.getLogger("mteb").setLevel(logging.INFO)
    elif args.verbosity == 3:
        logging.getLogger("mteb").setLevel(logging.DEBUG)

    logger.info("Running with parameters: %s", args)

    model = mteb.get_model(args.model, args.model_revision, device=args.device)

    tasks = mteb.get_tasks(
        categories=args.categories,
        task_types=args.task_types,
        languages=args.languages,
        tasks=args.tasks,
    )
    eval = mteb.MTEB(tasks=tasks)

    eval.run(
        model,
        verbosity=args.verbosity,
        output_folder=args.output_folder,
        eval_splits=args.eval_splits,
        co2_tracker=args.co2_tracker,
    )

    _save_model_metadata(model, Path(args.output_folder))


def available_tasks(args: argparse.Namespace) -> None:
    tasks = mteb.get_tasks(
        categories=args.categories,
        task_types=args.task_types,
        languages=args.languages,
        tasks=args.tasks,
    )
    eval = mteb.MTEB(tasks=tasks)
    eval.mteb_tasks()


def add_task_selection_args(parser: argparse.ArgumentParser) -> None:
    """Adds arguments to the parser for filtering tasks by type, category, language, and task name."""
    parser.add_argument(
        "--task_types",
        nargs="+",
        type=str,
        default=None,
        help="List of task types (Clustering, Retrieval..) to be evaluated. If None, the filter is not applied",
    )
    parser.add_argument(
        "--categories",
        nargs="+",
        type=str,
        default=None,
        help="List of task categories (s2s, p2p..) to be evaluated. If None the filter is not applied",
    )
    parser.add_argument(
        "-t",
        "--tasks",
        nargs="+",
        type=str,
        default=None,
        help="List of tasks to be evaluated. If specified, the other arguments are ignored.",
    )
    parser.add_argument(
        "-l",
        "--languages",
        nargs="*",
        type=str,
        default=None,
        help="List of languages to be evaluated. if not set, all languages will be evaluated. Specified as ISO 639-3 codes (e.g. eng, deu, fra).",
    )


def add_available_tasks_parser(subparsers) -> None:
    parser = subparsers.add_parser(
        "available_tasks", help="List the available tasks within MTEB"
    )
    add_task_selection_args(parser)

    parser.set_defaults(func=available_tasks)


def add_run_parser(subparsers) -> None:
    parser = subparsers.add_parser("run", help="Run a model on a set of tasks")

    parser.add_argument(
        "-m",
        "--model",
        type=str,
        help="Model to use. Will priotize model implementation in MTEB's model registry, but default to loading the model using sentence-transformers.",
    )

    add_task_selection_args(parser)

    parser.add_argument(
        "--device", type=int, default=None, help="Device to use for computation"
    )
    parser.add_argument(
        "--output_folder",
        type=str,
        default="results",
        help="Output directory for results. Will default to `results` if not set.",
    )
    parser.add_argument(
        "-v", "--verbosity", type=int, default=2, help="Verbosity level"
    )
    parser.add_argument(
        "--co2_tracker",
        type=bool,
        default=False,
        help="Enable CO₂ tracker, disabled by default",
    )
    parser.add_argument(
        "--eval_splits",
        nargs="+",
        type=str,
        default=None,
        help="Evaluation splits to use (train, dev, test..). If None, all splits will be used",
    )
    parser.add_argument(
        "--model_revision",
        type=str,
        default=None,
        help="Revision of the model to be loaded. Revisions are automatically read if the model is loaded from huggingface.",
    )

    parser.set_defaults(func=run)


def create_meta(args: argparse.Namespace) -> None:
    results_folder = Path(args.results_folder)
    output_path = Path(args.output_path)

    if output_path.exists() and args.overwrite:
        logger.warning("Output path already exists, overwriting.")
    elif output_path.exists():
        raise FileExistsError(
            "Output path already exists, use --overwrite to overwrite."
        )

    json_files = [
        r
        for r in results_folder.glob("*.json")
        if r.is_file() and r.name != "model_meta.json"
    ]

    task_results = [MTEBResults.from_disk(path) for path in json_files]
    task_results = sorted(task_results, key=lambda x: x.task_name)

    yaml_results = []
    for task_result in task_results:
        task = mteb.get_task(task_result.task_name)

        for split, hf_subset_scores in task_result.scores.items():
            for hf_subset_score in hf_subset_scores:
                metrics = [
                    {
                        "type": k,
                        "value": v
                        * 100,  # convert to percentage (for consistency with the leaderboard and to make it more readable)
                    }
                    for k, v in hf_subset_score.items()
                    if isinstance(v, (int, float))
                ]
                if task.metadata.main_score not in hf_subset_score:
                    raise ValueError(
                        f"Main score {task.metadata.main_score} not found in metrics or is not a number."
                    )

                yaml_result = {
                    "task": {"type": task.metadata.type},
                    "dataset": {
                        "type": task.metadata.dataset["path"],
                        "name": f"MTEB {task.metadata.name} ({hf_subset_score['hf_subset']})",
                        "config": hf_subset_score["hf_subset"],
                        "split": split,
                        "revision": task_result.dataset_revision,
                    },
                    "metrics": metrics,
                }
                yaml_results.append(yaml_result)

    model_name = "PLACEHOLDER"
    # if model_meta.json exists, use the model name from there
    if (results_folder / "model_meta.json").exists():
        with (results_folder / "model_meta.json").open("r") as f:
            model_meta = json.load(f)
            model_name = model_meta["name"]

    yaml_dict = {
        "tags": ["mteb"],
        "model-index": [
            {
                "name": model_name,
                # should we add the revision here?
                "results": yaml_results,
            }
        ],
    }

    with output_path.open("w") as f:
        yaml_str = yaml.dump(yaml_dict)
        frontmatter = "---\n" + yaml_str + "---\n"
        f.write(frontmatter)


def add_create_meta_parser(subparsers) -> None:
    parser = subparsers.add_parser(
        "create_meta", help="Create model metadata from a folder of results"
    )

    parser.add_argument(
        "--results_folder",
        type=str,
        help="Folder containing the results of a model run",
    )
    parser.add_argument(
        "--output_path",
        type=str,
        default="model_card.md",
        help="Output path for the model metadata",
    )
    parser.add_argument(
        "--overwrite",
        action="store_true",
        default=False,
        help="Overwrite the output file if it already exists",
    )

    parser.set_defaults(func=create_meta)


def main():
    parser = argparse.ArgumentParser(description="The MTEB Command line interface.")

    subparsers = parser.add_subparsers(
        title="subcommands", description="valid subcommands", help="additional help"
    )
    add_run_parser(subparsers)
    add_available_tasks_parser(subparsers)
    add_create_meta_parser(subparsers)

    args = parser.parse_args()

    # If no subcommand is provided, default to run with a deprecation warning
    if not hasattr(args, "func"):
        logger.warning(
            "Using `mteb` without a subcommand is deprecated. Use `mteb run` instead.",
            DeprecationWarning,
        )
        # Set default arguments for 'run' if no subcommand is provided
        default_args = parser.parse_args(
            ["run"]
            + list(map(str, args._get_args()))
            + [
                f"--{k}" if v is None else f"--{k}={v}"
                for k, v in vars(args).items()
                if k != "func"
            ]
        )
        default_args.func(default_args)
    else:
        args.func(args)


if __name__ == "__main__":
    main()



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class DBPedia(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="DBPedia",
        description="DBpedia-Entity is a standard test collection for entity search over the DBpedia knowledge base",
        reference="https://github.com/iai-group/DBpedia-Entity/",
        dataset={
            "path": "mteb/dbpedia",
            "revision": "c0f706b76e590d620bd6618b3ca8efdd34e2d659",
        },
        type="Retrieval",
        category="s2p",
        eval_splits=["dev", "test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=None,
        form=None,
        domains=None,
        task_subtypes=None,
        license=None,
        socioeconomic_status=None,
        annotations_creators=None,
        dialect=None,
        text_creation=None,
        bibtex_citation="""@inproceedings{Hasibi:2017:DVT,
 author =    {Hasibi, Faegheh and Nikolaev, Fedor and Xiong, Chenyan and Balog, Krisztian and Bratsberg, Svein Erik and Kotov, Alexander and Callan, Jamie},
 title =     {DBpedia-Entity V2: A Test Collection for Entity Search},
 booktitle = {Proceedings of the 40th International ACM SIGIR Conference on Research and Development in Information Retrieval},
 series =    {SIGIR '17},
 year =      {2017},
 pages =     {1265--1268},
 doi =       {10.1145/3077136.3080751},
 publisher = {ACM}
}""",
        n_samples=None,
        avg_character_length={
            "test": {
                "average_document_length": 1122.7690155333814,
                "average_query_length": 48.7264325323475,
                "num_documents": 48605,
                "num_queries": 541,
                "average_relevant_docs_per_query": 1.3752310536044363,
            }
        },
    )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class SciFact(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SciFact",
        dataset={
            "path": "mteb/scifact",
            "revision": "0228b52cf27578f30900b9e5271d331663a030d7",
        },
        description="SciFact verifies scientific claims using evidence from the research literature containing scientific paper abstracts.",
        reference="https://github.com/allenai/scifact",
        type="Retrieval",
        category="s2p",
        eval_splits=["train", "test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=None,
        form=None,
        domains=None,
        task_subtypes=None,
        license=None,
        socioeconomic_status=None,
        annotations_creators=None,
        dialect=None,
        text_creation=None,
        bibtex_citation="""@inproceedings{specter2020cohan,
  title={SPECTER: Document-level Representation Learning using Citation-informed Transformers},
  author={Arman Cohan and Sergey Feldman and Iz Beltagy and Doug Downey and Daniel S. Weld},
  booktitle={ACL},
  year={2020}
}""",
        n_samples=None,
        avg_character_length={
            "train": {
                "average_document_length": 1498.4152035500674,
                "average_query_length": 88.58838071693448,
                "num_documents": 5183,
                "num_queries": 809,
                "average_relevant_docs_per_query": 1.1359703337453646,
            },
            "test": {
                "average_document_length": 1498.4152035500674,
                "average_query_length": 90.34666666666666,
                "num_documents": 5183,
                "num_queries": 300,
                "average_relevant_docs_per_query": 1.13,
            },
        },
    )



import csv

from huggingface_hub import snapshot_download

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class MLQuestionsRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="MLQuestions",
        dataset={
            "path": "McGill-NLP/mlquestions",
            "revision": "83b690cb666c5a8869e7f213a877bbd24a642d7c",
        },
        reference="https://github.com/McGill-NLP/MLQuestions",
        description=(
            "MLQuestions is a domain adaptation dataset for the machine learning domain"
            "It consists of ML questions along with passages from Wikipedia machine learning pages (https://en.wikipedia.org/wiki/Category:Machine_learning)"
        ),
        type="Retrieval",
        category="s2p",
        eval_splits=["dev", "test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=(
            "2021-01-01",
            "2021-03-31",
        ),  # The period here is for both wiki articles and queries
        form=["written"],
        domains=["Encyclopaedic", "Academic"],
        task_subtypes=["Question answering"],
        license="cc-by-nc-sa-4.0",
        socioeconomic_status="mixed",
        annotations_creators="human-annotated",
        dialect=[],
        text_creation="found",
        bibtex_citation="""
            @inproceedings{kulshreshtha-etal-2021-back,
                title = "Back-Training excels Self-Training at Unsupervised Domain Adaptation of Question Generation and Passage Retrieval",
                author = "Kulshreshtha, Devang  and
                Belfer, Robert  and
                Serban, Iulian Vlad  and
                Reddy, Siva",
                booktitle = "Proceedings of the 2021 Conference on Empirical Methods in Natural Language Processing",
                month = nov,
                year = "2021",
                address = "Online and Punta Cana, Dominican Republic",
                publisher = "Association for Computational Linguistics",
                url = "https://aclanthology.org/2021.emnlp-main.566",
                pages = "7064--7078",
                abstract = "In this work, we introduce back-training, an alternative to self-training for unsupervised domain adaptation (UDA). While self-training generates synthetic training data where natural inputs are aligned with noisy outputs, back-training results in natural outputs aligned with noisy inputs. This significantly reduces the gap between target domain and synthetic data distribution, and reduces model overfitting to source domain. We run UDA experiments on question generation and passage retrieval from the Natural Questions domain to machine learning and biomedical domains. We find that back-training vastly outperforms self-training by a mean improvement of 7.8 BLEU-4 points on generation, and 17.6{\%} top-20 retrieval accuracy across both domains. We further propose consistency filters to remove low-quality synthetic data before training. We also release a new domain-adaptation dataset - MLQuestions containing 35K unaligned questions, 50K unaligned passages, and 3K aligned question-passage pairs.",
            }
        """,
        n_samples={"dev": 1500, "test": 1500},
        avg_character_length={
            "dev": {
                "average_document_length": 258.8772727272727,
                "average_query_length": 45.05533333333333,
                "num_documents": 11000,
                "num_queries": 1500,
                "average_relevant_docs_per_query": 1.0,
            },
            "test": {
                "average_document_length": 258.8772727272727,
                "average_query_length": 45.75333333333333,
                "num_documents": 11000,
                "num_queries": 1500,
                "average_relevant_docs_per_query": 1.0,
            },
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return
        self.corpus, self.queries, self.relevant_docs = {}, {}, {}
        dataset_path = self.metadata_dict["dataset"]["path"]
        revision = self.metadata_dict["dataset"].get("revision", None)
        download_dir = snapshot_download(
            repo_id=dataset_path, repo_type="dataset", revision=revision
        )
        for split in kwargs.get("eval_splits", self.metadata_dict["eval_splits"]):
            corpus, queries, qrels = self._load_data_for_split(download_dir, split)
            self.corpus[split], self.queries[split], self.relevant_docs[split] = (
                corpus,
                queries,
                qrels,
            )

        self.data_loaded = True

    def _load_data_for_split(self, download_dir, split):
        queries, corpus, qrels = {}, {}, {}

        dataset_path = f"{download_dir}/{split}.csv"
        with open(dataset_path, "r") as csvfile:
            reader = csv.DictReader(csvfile)
            for i, row in enumerate(reader):
                query_id = str(i)
                doc_id = row["indexes"]
                query = row["target_text"]
                queries[query_id] = query
                qrels[query_id] = {doc_id: 1}

        # Same corpus for all splits
        corpus_path = f"{download_dir}/test_passages.csv"
        with open(corpus_path, "r") as csvfile:
            reader = csv.DictReader(csvfile)
            for i, row in enumerate(reader):
                doc_id = str(i)
                corpus[doc_id] = {
                    "title": "",
                    "text": row["input_text"],
                }

        return corpus, queries, qrels



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class SpartQA(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SpartQA",
        description="Measuring the ability to retrieve the groundtruth answers to reasoning task queries on SpartQA.",
        reference="https://github.com/HLR/SpartQA_generation",
        dataset={
            "path": "RAR-b/spartqa",
            "revision": "9ab3ca3ccdd0d43f9cd6d346a363935d127f4f45",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2s",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2021-01-01", "2021-12-31"),
        form=["written"],
        domains=["Encyclopaedic"],
        task_subtypes=["Reasoning as Retrieval"],
        license="MIT",
        socioeconomic_status="medium",
        annotations_creators="derived",
        dialect=[],
        text_creation="found",
        bibtex_citation="""@article{xiao2024rar,
  title={RAR-b: Reasoning as Retrieval Benchmark},
  author={Xiao, Chenghao and Hudson, G Thomas and Moubayed, Noura Al},
  journal={arXiv preprint arXiv:2404.06347},
  year={2024}
}
@article{mirzaee2021spartqa,
  title={Spartqa:: A textual question answering benchmark for spatial reasoning},
  author={Mirzaee, Roshanak and Faghihi, Hossein Rajaby and Ning, Qiang and Kordjmashidi, Parisa},
  journal={arXiv preprint arXiv:2104.05832},
  year={2021}
}
""",
        n_samples={"test": 0},
        avg_character_length={
            "test": {
                "average_document_length": 50.40829145728643,
                "average_query_length": 656.2328881469115,
                "num_documents": 1592,
                "num_queries": 3594,
                "average_relevant_docs_per_query": 1.8786867000556482,
            }
        },
    )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class RARbMath(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="RARbMath",
        description="Measuring the ability to retrieve the groundtruth answers to reasoning task queries on RAR-b math-pooled dataset.",
        reference="https://arxiv.org/abs/2404.06347",
        dataset={
            "path": "RAR-b/math-pooled",
            "revision": "2393603c0221ff52f448d12dd75f0856103c6cca",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2p",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2021-01-01", "2023-12-31"),
        form=["written"],
        domains=["Encyclopaedic"],
        task_subtypes=["Reasoning as Retrieval"],
        license="MIT",
        socioeconomic_status="medium",
        annotations_creators="derived",
        dialect=[],
        text_creation="found",
        bibtex_citation="""@article{xiao2024rar,
  title={RAR-b: Reasoning as Retrieval Benchmark},
  author={Xiao, Chenghao and Hudson, G Thomas and Moubayed, Noura Al},
  journal={arXiv preprint arXiv:2404.06347},
  year={2024}
}
@article{hendrycks2021measuring,
  title={Measuring mathematical problem solving with the math dataset},
  author={Hendrycks, Dan and Burns, Collin and Kadavath, Saurav and Arora, Akul and Basart, Steven and Tang, Eric and Song, Dawn and Steinhardt, Jacob},
  journal={arXiv preprint arXiv:2103.03874},
  year={2021}
}
@article{cobbe2021training,
  title={Training verifiers to solve math word problems},
  author={Cobbe, Karl and Kosaraju, Vineet and Bavarian, Mohammad and Chen, Mark and Jun, Heewoo and Kaiser, Lukasz and Plappert, Matthias and Tworek, Jerry and Hilton, Jacob and Nakano, Reiichiro and others},
  journal={arXiv preprint arXiv:2110.14168},
  year={2021}
}
@article{yu2023metamath,
  title={Metamath: Bootstrap your own mathematical questions for large language models},
  author={Yu, Longhui and Jiang, Weisen and Shi, Han and Yu, Jincheng and Liu, Zhengying and Zhang, Yu and Kwok, James T and Li, Zhenguo and Weller, Adrian and Liu, Weiyang},
  journal={arXiv preprint arXiv:2309.12284},
  year={2023}
}
""",
        n_samples={"test": 6319},
        avg_character_length={
            "test": {
                "average_document_length": 504.0197829347469,
                "average_query_length": 210.30732710871973,
                "num_documents": 389376,
                "num_queries": 6319,
                "average_relevant_docs_per_query": 1.0,
            }
        },
    )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class NQ(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="NQ",
        dataset={
            "path": "mteb/nq",
            "revision": "b774495ed302d8c44a3a7ea25c90dbce03968f31",
        },
        description="NFCorpus: A Full-Text Learning to Rank Dataset for Medical Information Retrieval",
        reference="https://ai.google.com/research/NaturalQuestions/",
        type="Retrieval",
        category="s2p",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=None,
        form=None,
        domains=None,
        task_subtypes=None,
        license=None,
        socioeconomic_status=None,
        annotations_creators=None,
        dialect=None,
        text_creation=None,
        bibtex_citation="""@article{47761,title	= {Natural Questions: a Benchmark for Question Answering Research},
        author	= {Tom Kwiatkowski and Jennimaria Palomaki and Olivia Redfield and Michael Collins and Ankur Parikh 
        and Chris Alberti and Danielle Epstein and Illia Polosukhin and Matthew Kelcey and Jacob Devlin and Kenton Lee 
        and Kristina N. Toutanova and Llion Jones and Ming-Wei Chang and Andrew Dai and Jakob Uszkoreit and Quoc Le 
        and Slav Petrov},year	= {2019},journal	= {Transactions of the Association of Computational 
        Linguistics}}""",
        n_samples=None,
        avg_character_length={
            "test": {
                "average_document_length": 492.2287851281462,
                "average_query_length": 48.17902665121669,
                "num_documents": 2681468,
                "num_queries": 3452,
                "average_relevant_docs_per_query": 1.2169756662804172,
            }
        },
    )



from __future__ import annotations

import heapq
import json
import logging
import os
from collections import defaultdict
from typing import Dict, List, Tuple, Union

import numpy as np
import pytrec_eval
import torch
import tqdm
from sentence_transformers import CrossEncoder, SentenceTransformer
from sentence_transformers.models import Transformer, WordEmbeddings

from mteb.encoder_interface import EncoderWithQueryCorpusEncode

from .Evaluator import Evaluator
from .model_encode import model_encode
from .utils import (
    confidence_scores,
    convert_conv_history_to_query,
    cos_sim,
    dot_score,
    download,
    hole,
    mrr,
    nAUC,
    recall_cap,
    top_k_accuracy,
)

logger = logging.getLogger(__name__)


# Adapted from https://github.com/beir-cellar/beir/blob/f062f038c4bfd19a8ca942a9910b1e0d218759d4/beir/retrieval/search/dense/exact_search.py#L12
class DenseRetrievalExactSearch:
    def __init__(
        self,
        model: EncoderWithQueryCorpusEncode,
        batch_size: int = 128,
        corpus_chunk_size: int = 50000,
        previous_results: str | None = None,
        **kwargs,
    ):
        # Model is class that provides encode_corpus() and encode_queries()
        self.model = model
        self.batch_size = batch_size
        self.score_functions = {"cos_sim": cos_sim, "dot": dot_score}
        self.score_function_desc = {
            "cos_sim": "Cosine Similarity",
            "dot": "Dot Product",
        }
        self.corpus_chunk_size = corpus_chunk_size
        self.previous_results = previous_results
        self.show_progress_bar = kwargs.get("show_progress_bar", True)
        self.convert_to_tensor = kwargs.get("convert_to_tensor", True)
        self.save_corpus_embeddings = kwargs.get("save_corpus_embeddings", False)
        self.corpus_embeddings = defaultdict(list)
        self.results = {}

        if self.previous_results is not None:
            self.previous_results = self.load_results_file()

        if isinstance(self.model, CrossEncoder):
            # load the predict instance from the CrossEncoder
            # custom functions can be used by extending the DenseRetrievalExactSearch class
            self.predict = self.model.predict

    def search(
        self,
        corpus: dict[str, dict[str, str]],
        queries: dict[str, Union[str, List[str]]],
        top_k: int,
        score_function: str,
        prompt_name: str,
        return_sorted: bool = False,
        **kwargs,
    ) -> dict[str, dict[str, float]]:
        # Create embeddings for all queries using model.encode_queries()
        # Runs semantic search against the corpus embeddings
        # Returns a ranked list with the corpus ids
        if score_function not in self.score_functions:
            raise ValueError(
                f"score function: {score_function} must be either (cos_sim) for cosine similarity or (dot) for dot product"
            )

        logger.info("Encoding Queries.")
        query_ids = list(queries.keys())
        self.results = {qid: {} for qid in query_ids}
        queries = [queries[qid] for qid in queries]
        if isinstance(queries[0], list):
            query_embeddings = self.model.encode_conversations(
                queries,
                prompt_name=prompt_name,
                batch_size=self.batch_size,
                show_progress_bar=self.show_progress_bar,
                convert_to_tensor=self.convert_to_tensor,
                **kwargs,
            )
        else:
            query_embeddings = self.model.encode_queries(
                queries,
                prompt_name=prompt_name,
                batch_size=self.batch_size,
                show_progress_bar=self.show_progress_bar,
                convert_to_tensor=self.convert_to_tensor,
                **kwargs,
            )

        logger.info("Sorting Corpus by document length (Longest first)...")
        corpus_ids = sorted(
            corpus,
            key=lambda k: len(corpus[k].get("title", "") + corpus[k].get("text", "")),
            reverse=True,
        )
        corpus = [corpus[cid] for cid in corpus_ids]

        logger.info("Encoding Corpus in batches... Warning: This might take a while!")
        logger.info(
            "Scoring Function: {} ({})".format(
                self.score_function_desc[score_function], score_function
            )
        )

        itr = range(0, len(corpus), self.corpus_chunk_size)

        result_heaps = {
            qid: [] for qid in query_ids
        }  # Keep only the top-k docs for each query
        for batch_num, corpus_start_idx in enumerate(itr):
            logger.info("Encoding Batch {}/{}...".format(batch_num + 1, len(itr)))
            corpus_end_idx = min(corpus_start_idx + self.corpus_chunk_size, len(corpus))

            # Encode chunk of corpus
            if (
                self.save_corpus_embeddings
                and "qid" in kwargs
                and len(self.corpus_embeddings[kwargs["qid"]])
            ):
                sub_corpus_embeddings = torch.tensor(
                    self.corpus_embeddings[kwargs["qid"]][batch_num]
                )
            else:
                # Encode chunk of corpus
                sub_corpus_embeddings = self.model.encode_corpus(
                    corpus[corpus_start_idx:corpus_end_idx],  # type: ignore
                    prompt_name=prompt_name,
                    batch_size=self.batch_size,
                    show_progress_bar=self.show_progress_bar,
                    convert_to_tensor=self.convert_to_tensor,
                )
                if self.save_corpus_embeddings and "qid" in kwargs:
                    self.corpus_embeddings[kwargs["qid"]].append(sub_corpus_embeddings)

            # Compute similarites using either cosine-similarity or dot product
            cos_scores = self.score_functions[score_function](
                query_embeddings, sub_corpus_embeddings
            )
            cos_scores[torch.isnan(cos_scores)] = -1

            # Get top-k values
            cos_scores_top_k_values, cos_scores_top_k_idx = torch.topk(
                cos_scores,
                min(
                    top_k + 1,
                    len(cos_scores[1]) if len(cos_scores) > 1 else len(cos_scores[-1]),
                ),
                dim=1,
                largest=True,
                sorted=return_sorted,
            )
            cos_scores_top_k_values = cos_scores_top_k_values.cpu().tolist()
            cos_scores_top_k_idx = cos_scores_top_k_idx.cpu().tolist()

            for query_itr in range(len(query_embeddings)):
                query_id = query_ids[query_itr]
                for sub_corpus_id, score in zip(
                    cos_scores_top_k_idx[query_itr], cos_scores_top_k_values[query_itr]
                ):
                    corpus_id = corpus_ids[corpus_start_idx + sub_corpus_id]
                    if len(result_heaps[query_id]) < top_k:
                        # Push item on the heap
                        heapq.heappush(result_heaps[query_id], (score, corpus_id))
                    else:
                        # If item is larger than the smallest in the heap, push it on the heap then pop the smallest element
                        heapq.heappushpop(result_heaps[query_id], (score, corpus_id))

        for qid in result_heaps:
            for score, corpus_id in result_heaps[qid]:
                self.results[qid][corpus_id] = score

        return self.results

    def load_results_file(self):
        # load the first stage results from file in format {qid: {doc_id: score}}
        if "https://" in self.previous_results:
            # download the file
            if not os.path.exists(self.previous_results):
                url_descriptor = self.previous_results.split("https://")[-1].replace(
                    "/", "--"
                )
                dest_file = os.path.join(
                    "results", f"cached_predictions--{url_descriptor}"
                )
                os.makedirs(os.path.dirname(os.path.abspath(dest_file)), exist_ok=True)
                download(self.previous_results, dest_file)
                logger.info(
                    f"Downloaded the previous results at {self.previous_results} to {dest_file}"
                )
            self.previous_results = dest_file

        with open(self.previous_results, "r") as f:
            previous_results = json.load(f)
        assert isinstance(previous_results, dict)
        assert isinstance(previous_results[list(previous_results.keys())[0]], dict)
        return previous_results

    def search_cross_encoder(
        self,
        corpus: Dict[str, Dict[str, str]],
        queries: Dict[str, Union[str, List[str]]],
        top_k: int,
        instructions: Dict[str, str] | None = None,
        **kwargs,
    ) -> Dict[str, Dict[str, float]]:
        """This function provides support for reranker (or cross-encoder) models that encoder query and document at the same time (typically with attention).
        Some notable examples include MonoBERT, MonoT5, RankLlama, etc.
        Note: you must provide the path to the results to rerank to the __init__ function as `previous_results`
        """
        pairs = []  # create the pairs for reranking
        for qid in queries.keys():
            q_results = self.previous_results[qid]
            # take the top-k only
            q_results_sorted = {
                k: v
                for k, v in sorted(
                    q_results.items(), key=lambda item: item[1], reverse=True
                )
            }
            top_n = [k for k, v in list(q_results_sorted.items())[:top_k]]
            query = queries[qid]
            query = (
                self.convert_conv_history_to_query([query])[0]
                if isinstance(query, list)
                else query
            )
            for doc_id in top_n:
                corpus_item = (
                    corpus[doc_id].get("title", "") + " " + corpus[doc_id]["text"]
                ).strip()
                pairs.append(
                    (
                        query,
                        corpus_item,
                        instructions[query] if instructions is not None else None,
                        qid,
                        doc_id,
                    )
                )

        logger.info(f"Reranking the top {top_k} in batches... This might take a while!")
        itr = range(0, len(pairs), self.batch_size)

        results = {qid: {} for qid in queries.keys()}
        for batch_num, corpus_start_idx in enumerate(
            tqdm.tqdm(itr, leave=False, disable=not self.show_progress_bar)
        ):
            corpus_end_idx = min(corpus_start_idx + self.batch_size, len(pairs))
            cur_batch = pairs[corpus_start_idx:corpus_end_idx]

            (
                queries_in_pair,
                corpus_in_pair,
                instructions_in_pair,
                query_ids,
                corpus_ids,
            ) = zip(*cur_batch)

            assert (
                len(queries_in_pair) == len(corpus_in_pair) == len(instructions_in_pair)
            )

            if isinstance(self.model, CrossEncoder):
                # can't take instructions, so add them here
                queries_in_pair = [
                    f"{q} {i}".strip()
                    for i, q in zip(instructions_in_pair, queries_in_pair)
                ]
                scores = self.model.predict(list(zip(queries_in_pair, corpus_in_pair)))
            else:
                # may use the instructions in a unique way, so give them also
                scores = self.model.predict(
                    list(zip(queries_in_pair, corpus_in_pair, instructions_in_pair))
                )

            for i, score in enumerate(scores):
                results[query_ids[i]][corpus_ids[i]] = float(score)

        return results

    def predict(self, queries, passages, **kwargs):
        raise NotImplementedError(
            "You must implement a predict method for your reranker model"
        )

    def encode_conversations(
        self, conversations: List[List[str]], prompt_name: str, **kwargs
    ):
        if callable(getattr(self.model, "encode_conversations", None)):
            return self.model.encode_conversations(
                conversations, prompt_name=prompt_name, **kwargs
            )
        # otherwise fallback to default implementation
        # TODO: add a warning here
        queries = self.convert_conv_history_to_query(conversations)
        return self.encode_queries(queries, prompt_name=prompt_name, **kwargs)

    def convert_conv_history_to_query(self, conversations: List[List[str]]) -> str:
        if callable(getattr(self.model, "convert_conv_history_to_query", None)):
            return self.model.convert_conv_history_to_query(conversations)
        return convert_conv_history_to_query(conversations)


class DRESModel:
    """Dense Retrieval Exact Search (DRES) requires an encode_queries & encode_corpus method.
    This class converts a model with just an .encode method into DRES format.
    """

    def __init__(self, model, **kwargs):
        self.model = model
        self.use_sbert_model = isinstance(model, SentenceTransformer)
        self.save_corpus_embeddings = kwargs.get("save_corpus_embeddings", False)
        self.corpus_embeddings = {}

    def encode_queries(
        self, queries: List[str], *, prompt_name: str, batch_size: int, **kwargs
    ):
        if self.use_sbert_model:
            if isinstance(self.model._first_module(), Transformer):
                logger.info(
                    f"Queries will be truncated to {self.model.get_max_seq_length()} tokens."
                )
            elif isinstance(self.model._first_module(), WordEmbeddings):
                logger.warning(
                    "Queries will not be truncated. This could lead to memory issues. In that case please lower the batch_size."
                )

        if "instructions" in kwargs:
            if kwargs["instructions"] is not None:
                queries = [
                    (query + " " + kwargs["instructions"][query]).strip()
                    for query in queries
                ]
            new_kwargs = {
                k: v for k, v in kwargs.items() if k not in ["instructions", "qid"]
            }
        else:
            # can't just delete, cuz assign by reference on kwargs
            new_kwargs = kwargs

        return model_encode(
            queries,
            model=self.model,
            prompt_name=prompt_name,
            batch_size=batch_size,
            **new_kwargs,
        )

    def encode_corpus(
        self, corpus: List[Dict[str, str]], prompt_name: str, batch_size: int, **kwargs
    ):
        if (
            "qid" in kwargs
            and self.save_corpus_embeddings
            and len(self.corpus_embeddings) > 0
        ):
            return self.corpus_embeddings[kwargs["qid"]]

        if isinstance(corpus, dict):
            sentences = [
                (corpus["title"][i] + " " + corpus["text"][i]).strip()
                if "title" in corpus
                else corpus["text"][i].strip()
                for i in range(len(corpus["text"]))
            ]
        else:
            sentences = [
                (doc["title"] + " " + doc["text"]).strip()
                if "title" in doc
                else doc["text"].strip()
                for doc in corpus
            ]

        if "instructions" in kwargs:  # not used on the doc side
            new_kwargs = {
                k: v for k, v in kwargs.items() if k not in ["instructions", "qid"]
            }
        else:
            # can't just delete, cuz assign by reference on kwargs
            new_kwargs = kwargs

        corpus_embeddings = model_encode(
            sentences,
            model=self.model,
            prompt_name=prompt_name,
            batch_size=batch_size,
            **new_kwargs,
        )

        if self.save_corpus_embeddings and "qid" in kwargs:
            self.corpus_embeddings[kwargs["qid"]] = corpus_embeddings
        return corpus_embeddings

    def encode(self, sentences: List[str], prompt_name: str, **kwargs):
        return self.encode_queries(sentences, prompt_name=prompt_name, **kwargs)

    def encode_conversations(
        self,
        conversations: List[List[str]],
        *,
        batch_size: int,
        prompt_name: str,
        **kwargs,
    ):
        if callable(getattr(self.model, "encode_conversations", None)):
            return self.model.encode_conversations(
                conversations, prompt_name=prompt_name, **kwargs
            )
        # otherwise fallback to default implementation
        # TODO: add a warning here
        queries = self.convert_conv_history_to_query(conversations)
        return self.encode_queries(
            queries, batch_size=batch_size, prompt_name=prompt_name, **kwargs
        )

    def convert_conv_history_to_query(self, conversations: List[List[str]]) -> str:
        if callable(getattr(self.model, "convert_conv_history_to_query", None)):
            return self.model.convert_conv_history_to_query(conversations)
        return convert_conv_history_to_query(conversations)


def is_dres_compatible(model):
    for method in ["encode_queries", "encode_corpus"]:
        op = getattr(model, method, None)
        if not (callable(op)):
            return False
    return True


def is_cross_encoder_compatible(model):
    op = getattr(model, "predict", None)
    if not (callable(op)):
        return False
    return True


# Adapted from https://github.com/beir-cellar/beir/blob/f062f038c4bfd19a8ca942a9910b1e0d218759d4/beir/retrieval/evaluation.py#L9
class RetrievalEvaluator(Evaluator):
    def __init__(
        self,
        retriever=None,
        task_name: str | None = None,
        k_values: List[int] = [1, 3, 5, 10, 20, 100, 1000],
        score_function: str = "cos_sim",
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.is_cross_encoder = False
        if is_cross_encoder_compatible(retriever):
            logger.info(
                "The custom predict function of the model will be used if not a SentenceTransformer CrossEncoder"
            )
            self.retriever = DenseRetrievalExactSearch(retriever, **kwargs)
            self.is_cross_encoder = True
        elif is_dres_compatible(retriever):
            logger.info(
                "The custom encode_queries and encode_corpus functions of the model will be used"
            )
            self.retriever = DenseRetrievalExactSearch(retriever, **kwargs)
        else:
            logger.info(
                "The model does not have the optional encode_queries and encode_corpus functions. Wrapping it in DRESModel."
            )
            self.retriever = DenseRetrievalExactSearch(DRESModel(retriever), **kwargs)
        self.k_values = k_values
        self.top_k = (
            max(k_values) if "top_k" not in kwargs else kwargs["top_k"]
        )  # can lower it if reranking
        self.score_function = score_function
        self.task_name = task_name

    def __call__(
        self,
        corpus: dict[str, dict[str, str]],
        queries: dict[str, Union[str, List[str]]],
    ) -> dict[str, dict[str, float]]:
        if not self.retriever:
            raise ValueError("Model/Technique has not been provided!")

        if self.is_cross_encoder:
            return self.retriever.search_cross_encoder(corpus, queries, self.top_k)
        else:
            return self.retriever.search(
                corpus,
                queries,
                self.top_k,
                self.score_function,
                prompt_name=self.task_name,
            )

    @staticmethod
    def evaluate(
        qrels: dict[str, dict[str, int]],
        results: dict[str, dict[str, float]],
        k_values: List[int],
        ignore_identical_ids: bool = False,
    ) -> Tuple[
        dict[str, float],
        dict[str, float],
        dict[str, float],
        dict[str, float],
        dict[str, float],
    ]:
        if ignore_identical_ids:
            logger.info(
                "For evaluation, ``ignore_identical_ids=True`` is set to True, the evaluator will ignore identical query and document ids."
            )
            # Remove identical ids from results dict
            for qid, rels in results.items():
                for pid in list(rels):
                    if qid == pid:
                        results[qid].pop(pid)
        else:
            logger.info(
                "For evaluation, we DO NOT ignore identical query and document ids (default), please explicitly set ``ignore_identical_ids=True`` to ignore this."
            )

        all_ndcgs, all_aps, all_recalls, all_precisions = {}, {}, {}, {}

        for k in k_values:
            all_ndcgs[f"NDCG@{k}"] = []
            all_aps[f"MAP@{k}"] = []
            all_recalls[f"Recall@{k}"] = []
            all_precisions[f"P@{k}"] = []

        map_string = "map_cut." + ",".join([str(k) for k in k_values])
        ndcg_string = "ndcg_cut." + ",".join([str(k) for k in k_values])
        recall_string = "recall." + ",".join([str(k) for k in k_values])
        precision_string = "P." + ",".join([str(k) for k in k_values])
        evaluator = pytrec_eval.RelevanceEvaluator(
            qrels, {map_string, ndcg_string, recall_string, precision_string}
        )
        scores = evaluator.evaluate(results)

        for query_id in scores.keys():
            for k in k_values:
                all_ndcgs[f"NDCG@{k}"].append(scores[query_id]["ndcg_cut_" + str(k)])
                all_aps[f"MAP@{k}"].append(scores[query_id]["map_cut_" + str(k)])
                all_recalls[f"Recall@{k}"].append(scores[query_id]["recall_" + str(k)])
                all_precisions[f"P@{k}"].append(scores[query_id]["P_" + str(k)])

        ndcg, _map, recall, precision = (
            all_ndcgs.copy(),
            all_aps.copy(),
            all_recalls.copy(),
            all_precisions.copy(),
        )

        for k in k_values:
            ndcg[f"NDCG@{k}"] = round(sum(ndcg[f"NDCG@{k}"]) / len(scores), 5)
            _map[f"MAP@{k}"] = round(sum(_map[f"MAP@{k}"]) / len(scores), 5)
            recall[f"Recall@{k}"] = round(sum(recall[f"Recall@{k}"]) / len(scores), 5)
            precision[f"P@{k}"] = round(sum(precision[f"P@{k}"]) / len(scores), 5)

        naucs = RetrievalEvaluator.evaluate_abstention(
            results, {**all_ndcgs, **all_aps, **all_recalls, **all_precisions}
        )

        return ndcg, _map, recall, precision, naucs

    @staticmethod
    def evaluate_custom(
        qrels: dict[str, dict[str, int]],
        results: dict[str, dict[str, float]],
        k_values: List[int],
        metric: str,
        output_type: str = "all",
    ) -> Tuple[Dict[str, float]]:
        if metric.lower() in ["mrr", "mrr@k", "mrr_cut"]:
            metric_scores = mrr(qrels, results, k_values, output_type)

        elif metric.lower() in ["recall_cap", "r_cap", "r_cap@k"]:
            metric_scores = recall_cap(qrels, results, k_values, output_type)

        elif metric.lower() in ["hole", "hole@k"]:
            metric_scores = hole(qrels, results, k_values, output_type)

        elif metric.lower() in [
            "acc",
            "top_k_acc",
            "accuracy",
            "accuracy@k",
            "top_k_accuracy",
        ]:
            metric_scores = top_k_accuracy(qrels, results, k_values, output_type)

        naucs = RetrievalEvaluator.evaluate_abstention(results, metric_scores)
        metric_scores_avg = {k: sum(v) / len(v) for k, v in metric_scores.items()}

        return metric_scores_avg, naucs

    @staticmethod
    def evaluate_abstention(
        results: dict[str, dict[str, float]],
        metric_scores: dict[str, list[float]],
    ) -> Dict[str, float]:
        """Computes normalized Area Under the Curve on a set of evaluated instances as presented in the paper https://arxiv.org/abs/2402.12997"""
        all_sim_scores = [list(results[qid].values()) for qid in list(results.keys())]
        all_conf_scores = [
            confidence_scores(sim_scores) for sim_scores in all_sim_scores
        ]
        conf_fcts = list(all_conf_scores[0].keys())
        all_conf_scores = {
            fct: np.array([x[fct] for x in all_conf_scores]) for fct in conf_fcts
        }
        metric_scores = {k: np.array(v) for k, v in metric_scores.items()}
        naucs = {}

        for metric_name, scores in metric_scores.items():
            for fct, conf_scores in all_conf_scores.items():
                naucs[f"nAUC_{metric_name}_{fct}"] = nAUC(conf_scores, scores)

        return naucs



from __future__ import annotations

import datasets

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskReranking import AbsTaskReranking


class WebLINXCandidatesReranking(AbsTaskReranking):
    metadata = TaskMetadata(
        name="WebLINXCandidatesReranking",
        description="WebLINX is a large-scale benchmark of 100K interactions across 2300 expert demonstrations of conversational web navigation. The reranking task focuses on finding relevant elements at every given step in the trajectory.",
        reference="https://mcgill-nlp.github.io/weblinx",
        dataset={
            "path": "McGill-NLP/WebLINX",
            "name": "reranking",
            "revision": "ed1c933c2b3617e5700d8a7ebe07f5975969a453",
        },
        type="Reranking",
        category="p2p",
        eval_splits=[
            "validation",
            "test_iid",
            "test_cat",
            "test_geo",
            "test_vis",
            "test_web",
        ],
        eval_langs=["eng-Latn"],
        main_score="mrr",
        date=("2023-03-01", "2023-10-30"),
        form=["written"],
        domains=["Academic", "Web"],
        task_subtypes=["Code retrieval", "Conversational retrieval"],
        license="CC BY-NC-SA 4.0",
        socioeconomic_status="mixed",
        annotations_creators="expert-annotated",
        dialect=[],
        text_creation="created",
        bibtex_citation="""
@misc{lù2024weblinx,
      title={WebLINX: Real-World Website Navigation with Multi-Turn Dialogue}, 
      author={Xing Han Lù and Zdeněk Kasner and Siva Reddy},
      year={2024},
      eprint={2402.05930},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
        """,
        n_samples={
            "validation": 1301,
            "test_iid": 1438,
            "test_cat": 3560,
            "test_web": 3144,
            "test_vis": 5298,
            "test_geo": 4916,
        },
        avg_character_length={
            "validation": 1647.52,
            "test_iid": 1722.63,
            "test_cat": 2149.66,
            "test_web": 1831.46,
            "test_vis": 1737.26,
            "test_geo": 1742.66,
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self._datasets = {}

        for split in self.metadata.eval_splits:
            self._datasets[split] = datasets.load_dataset(
                split=split, **self.metadata_dict["dataset"]
            )

        self.dataset = datasets.DatasetDict(
            {split: self._datasets[split] for split in self.metadata.eval_splits}
        )

        self.dataset_transform()

        self.data_loaded = True
